@@ -1,10 +1,12 @@
 /*
  *
- * (c) Vladi Belperchinov-Shabanski "Cade" 1996-2000
- * http://www.biscom.net/~cade  <cade@biscom.net>  <cade@datamax.bg>
+ * (c) Vladi Belperchinov-Shabanski "Cade" 1996-2003
+ * http://soul.datamax.bg/~cade  <cade@biscom.net>  <cade@datamax.bg>
  *
  * SEE `README',`LICENSE' OR `COPYING' FILE FOR LICENSE AND OTHER DETAILS!
  *
+ * $Id: vfu.h,v 1.11 2003/04/28 17:17:01 cade Exp $
+ *
  */
 
 #ifndef _VFU_H_
@@ -16,6 +18,7 @@
   #include <sys/types.h>
   #include <sys/stat.h>
   #include <sys/wait.h>
+  #include <sys/param.h>
 
   #include <stdio.h>
   #include <stdlib.h>
@@ -32,8 +35,8 @@
 
   #include <vslib.h>
 
-  #ifdef _TARGET_NETBSD_
-    #include <sys/param.h>
+  #if defined(_TARGET_NETBSD_) || (defined(BSD) && BSD >= 199306)
+    //FIXME: is the one above correct? should _TARGET_BSD_ be used?
     #include <sys/mount.h>
   #else
     #include <sys/vfs.h>
@@ -63,6 +66,8 @@
 #endif
 
 #define FNMATCH(p,s) fnmatch((p),(s),FNMATCH_FLAGS)
+#define FNMATCH_NC(p,s) fnmatch((p),(s),FNM_CASEFOLD)
+#define FNMATCH_OC(p,s,n) fnmatch((p),(s),(n)?FNM_CASEFOLD:FNMATCH_FLAGS)
 
 #ifdef _TARGET_GO32_
   #define pathcmp strcasecmp
@@ -96,6 +101,7 @@
   #define HID_SEQ_SUFFIX 160
   #define HID_SEQ_DIGPOS 170
   #define HID_SEQ_START  180
+  #define HID_OMODE      190 // octal mode
 
   #define VFU_CHECK_LIST_POS(n) \
           { ASSERT( files_list[n] != NULL ); \
@@ -165,14 +171,14 @@
 
   /* work context */
   extern int    work_mode;
-  extern String work_path;
+  extern VString work_path;
   /* archive context */
-  extern String archive_name;
-  extern String archive_path;
-  extern PSZCluster archive_extensions;
+  extern VString archive_name;
+  extern VString archive_path;
+  extern VArray archive_extensions;
 
-  extern String external_panelizer;
-  extern PSZCluster list_panelizer;
+  extern VString external_panelizer;
+  extern VArray list_panelizer;
   
   extern TF*        files_list[MAX_FILES];
   /* file list statistics */
@@ -186,62 +192,65 @@
   extern fsize_t    fs_block_size;
   /* index in the files list */
   /* NOTE: following defines are kept for historical reasons :) */
-  extern  TScrollPos file_list_index; 
-  #define FLI       (file_list_index.pos)
-  #define FLP       (file_list_index.page)
-  #define PS        (file_list_index.pagesize)
-  #define FGO(n)    (file_list_index.gotopos(n))
+  extern  ScrollPos file_list_index; 
+  #define FLI       (file_list_index.pos())
+  #define FLP       (file_list_index.page())
+  #define PS        (file_list_index.pagesize())
+  #define FGO(n)    (file_list_index.go(n))
 
   /* some world wide variables */
-  extern String startup_path;
-  extern String home_path;
-  extern String tmp_path;
-  extern String rc_path;
+  extern VString startup_path;
+  extern VString home_path;
+  extern VString tmp_path;
+  extern VString rc_path;
   
   /* files masks */
-  extern String         files_mask;
-  extern StrSplitter    files_mask_array;
+  extern VString         files_mask;
+  extern VArray         files_mask_array;
 
 /*############################################ GLOBAL STRUCTS  #########*/
 
-  extern PSZCluster dir_tree;
+  extern VArray dir_tree;
   extern int        dir_tree_changed;
 
-  extern PSZCluster file_find_results; // filefind results
+  extern VArray file_find_results; // filefind results
 
-  extern String path_bookmarks[10];
+  extern VArray path_bookmarks;
 
 
 /*######################################################################*/
 
-  extern PSZCluster user_externals;
-  extern PSZCluster history;
-  extern PSZCluster see_filters;
-  extern PSZCluster panelizers;
+  extern VArray user_externals;
+  extern VArray history;
+  extern VArray see_filters;
+  extern VArray panelizers;
 
-  extern PSZCluster mb; /* menu boxes */
+  extern VArray mb; /* menu boxes */
   
-  extern StrSplitter trim_tree;
+  extern VArray trim_tree;
+
+  extern VArray view_profiles;
+  extern VString view_profile;
 
 /*############################################ CONFIG SETTINGS #########*/
 
-  extern String ext_colors[16];
+  extern VString ext_colors[16];
 
-  extern String shell_browser;
-  extern String shell_editor;
-  extern String shell_prog;
+  extern VString shell_browser;
+  extern VString shell_editor;
+  extern VString shell_prog;
 
-  extern String user_id_str;
-  extern String group_id_str;
-  extern String host_name_str;
+  extern VString user_id_str;
+  extern VString group_id_str;
+  extern VString host_name_str;
 
-  extern String filename_opt;
-  extern String filename_conf;
-  extern String filename_history;
-  extern String filename_tree;
-  extern String filename_size_cache;
-  extern String filename_ffr; /* file find results */
-  extern String filename_atl; /* bad name I know, it stands for
+  extern VString filename_opt;
+  extern VString filename_conf;
+  extern VString filename_history;
+  extern VString filename_tree;
+  extern VString filename_size_cache;
+  extern VString filename_ffr; /* file find results */
+  extern VString filename_atl; /* bad name I know, it stands for
                                  `archive temp list' */
 
 /*######################################################################*/
@@ -254,10 +263,14 @@
 /*
   Message issues
 */
+void say( int line, int attr, const char* format, ... );
+
 void say1(const char *a_str, int attr = cMESSAGE );
 void say2(const char *a_str, int attr = cMESSAGE );
 void say2errno();
 
+void saycenter( int line, int attr, const char *a_str );
+
 void say1center(const char *a_str, int attr = cMESSAGE );
 void say2center(const char *a_str, int attr = cMESSAGE );
 
@@ -272,6 +285,7 @@ void vfu_init();
 void vfu_run();
 void vfu_cli();
 void vfu_done();
+void vfu_reset_screen();
 void vfu_signal( int sig );
 void vfu_exit_path( const char *a_path );
 int vfu_exit( const char* a_path );
@@ -300,7 +314,7 @@ void bookmark_hookup();
 
 void update_status();
 
-int vfu_user_external_find( int key, const char* ext, const char* type, String *shell_line );
+int vfu_user_external_find( int key, const char* ext, const char* type, VString *shell_line );
 void vfu_user_external_exec( int a_key );
 void vfu_user_menu();
 
