@@ -1,10 +1,12 @@
 /*
  *
- * (c) Vladi Belperchinov-Shabanski "Cade" 1996-2000
- * http://www.biscom.net/~cade  <cade@biscom.net>  <cade@datamax.bg>
+ * (c) Vladi Belperchinov-Shabanski "Cade" 1996-2003
+ * http://soul.datamax.bg/~cade  <cade@biscom.net>  <cade@datamax.bg>
  *
  * SEE `README',`LICENSE' OR `COPYING' FILE FOR LICENSE AND OTHER DETAILS!
  *
+ * $Id: vfucopy.cpp,v 1.14 2003/05/06 10:56:04 cade Exp $
+ *
  */
 
 #include "vfu.h"
@@ -15,26 +17,40 @@
 #include "vfusys.h"
 #include "vfucopy.h"
 
+/****************************************************************************
+**
+** globals
+**
+****************************************************************************/
+
 char *CM_DESC[] = { "COPY", "MOVE", "LINK" };
 char *copy_buff = NULL;
 
-/////////////////////////////////////////////
-//
-// some utilities...
-//
+int ignore_copy_errors = 0; /* actually it is used for copy/move/erase */
+
+/* clipboard ***************************************************************/
+
+char *CB_DESC[] = { "COPY", "MOVE", "SYMLINK" };
+
+VTrie Clipboard;
+CopyInfo clipboard_copy_info;
+
+/****************************************************************************
+**
+** utilities
+**
+****************************************************************************/
 
 fsize_t device_free_space( const char *target ) /* user free space, NOT real! */
 {
-  char t[MAX_PATH];
   struct statfs stafs;
-  str_file_path( target, t );
-  statfs( t, &stafs );
+  statfs( str_file_path( target ), &stafs );
   return ((fsize_t)(stafs.f_bsize)) * stafs.f_bfree;
 };
 
 /*
-  return 0 if src and dst are actually the same file
-*/
+ *  return 0 if src and dst are actually the same file
+ */
 int file_is_same( const char *src, const char *dst ) 
 {
   #ifdef _TARGET_GO32_
@@ -153,7 +169,7 @@ int over_if_exist( const char* src, const char *dst, CopyInfo* copy_info )
   if ( copy_info->over_mode == OM_ALWAYS_IF_MTIME &&
        stat_src.st_mtime > stat_dst.st_mtime ) return 1; /* newer mtime, do it! */
 
-  String str;
+  VString str;
   char sttime[32];
 
   char s_t = (stat_src.st_mtime == stat_dst.st_mtime)?'*':' '; // same time
@@ -164,13 +180,13 @@ int over_if_exist( const char* src, const char *dst, CopyInfo* copy_info )
   time_str_compact( stat_src.st_mtime, sttime); 
   str = (long int)stat_src.st_size; 
   str_comma(str);
-  sprintf(t, "SRC: %s%c %11s%c %s", sttime, s_t, str.asis(), s_s, src );
+  sprintf(t, "SRC: %s%c %11s%c %s", sttime, s_t, str.data(), s_s, src );
   say1(t);
 
   time_str_compact(stat_dst.st_mtime, sttime); 
   str = (long int)stat_dst.st_size; 
   str_comma(str);
-  sprintf(t, "DST: %s%c %11s%c %s", sttime, s_t, str.asis(), s_s, dst );
+  sprintf(t, "DST: %s%c %11s%c %s", sttime, s_t, str.data(), s_s, dst );
   say2(t);
 
   vfu_beep();
@@ -217,11 +233,17 @@ int vfu_copy_mode( const char* src, const char* dst )
   return 0;  
 };
 
-/*###########################################################################*/
+/***************************************************************************
+**
+** COPY/MOVE/SYMLINK
+**
+****************************************************************************/
+
+/* copy/move ***************************************************************/
+
+
+//  return 0 for ok
 
-/*
-  return 0 for ok
-*/
 int __vfu_file_copy( const char* src, const char* dst, CopyInfo* copy_info )
 {
   errno = 0; /* clear error status */
@@ -243,8 +265,8 @@ int __vfu_file_copy( const char* src, const char* dst, CopyInfo* copy_info )
     }
 
   /* progress report */
-  String str = dst;
-  str_dot_reduce( NULL, str, con_max_x() - 10 );
+  VString str = dst;
+  str = str_dot_reduce( str, con_max_x() - 10 );
   str = "COPY TO: " + str;
   say1( str );
   con_out( 1, con_max_y(), copy_info->description, cMESSAGE );
@@ -370,31 +392,12 @@ int __vfu_file_move( const char* src, const char* dst, CopyInfo* copy_info )
 
 /*---------------------------------------------------------------------------*/
 
-int __vfu_file_symlink( const char* src, const char* dst, CopyInfo* copy_info )
-{
-  errno = 0; /* clear error status */
-  
-  if (!over_if_exist( src, dst, copy_info )) 
-    return copy_info->abort ? CR_ABORT : CR_SKIP; /* ok */
-  
-  if ( file_exist( dst ) )
-    {
-    if ( file_is_same( src, dst ) == 0 )
-      return CR_SKIP; /* dst is src actually */
-    /* FIXME: what if dst is symlink? */  
-    if ( __vfu_file_erase( dst ) ) return 1;
-    }
-  return (symlink( src, dst ) == -1);
-}
-
-/*###########################################################################*/
-
 int __vfu_dir_copy( const char* src, const char* dst, CopyInfo* copy_info )
 {
   errno = 0; /* clear error status */
   
-  String fname_src; /* used for directory entries */
-  String fname_dst; /* used for directory entries */
+  VString fname_src; /* used for directory entries */
+  VString fname_dst; /* used for directory entries */
 
   if ( vfu_break_op() ) 
     return CR_ABORT; /* canceled */
@@ -404,11 +407,19 @@ int __vfu_dir_copy( const char* src, const char* dst, CopyInfo* copy_info )
   
   if ( make_path( dst ) ) 
     {
+    if ( ignore_copy_errors ) return CR_ABORT; /* cancel operation */
+    
     say1( dst );
     say2errno();
-    vfu_menu_box( "Create dir error", "C Continue anyway,  Abort (ESC)", -1 );
+    vfu_menu_box( "Create dir error", 
+                  "C Continue anyway,I Ignore further errors,  Abort (ESC)", -1 );
     if ( menu_box_info.ec != 'C' )
       return CR_ABORT; /* cancel operation */
+    if ( menu_box_info.ec != 'I' )
+      {
+      ignore_copy_errors = 1;
+      return CR_ABORT; /* cancel operation */
+      }
     }
     
   DIR *dir;
@@ -432,14 +443,7 @@ int __vfu_dir_copy( const char* src, const char* dst, CopyInfo* copy_info )
     
     while(4)
       {
-      int r = 0;
-      if ( file_is_link( fname_src ) )
-        r = __vfu_link_copy( fname_src, fname_dst, copy_info ); /* symlink */
-      else
-      if ( file_is_dir( fname_src ) )  
-        r = __vfu_dir_copy( fname_src, fname_dst, copy_info ); /* directory */
-      else
-        r = __vfu_file_copy( fname_src, fname_dst, copy_info ); /* regular file */
+      int r = __vfu_copy( fname_src, fname_dst, copy_info );
 
       if ( r == CR_ABORT )
         {
@@ -448,15 +452,22 @@ int __vfu_dir_copy( const char* src, const char* dst, CopyInfo* copy_info )
         }
       if ( r && r != CR_SKIP )
         {
+        if ( ignore_copy_errors ) break;
+        
         say1( fname_dst );
         say2errno();
         vfu_menu_box( "Copy/Move/SymLink error", 
-                      "T Try again,S Skip/continue,  Abort (ESC)" );
+                      "T Try again,S Skip/continue,I Ignore further errors,  Abort (ESC)" );
         if ( menu_box_info.ec == 'T' )
           continue; /* while(4) */
         else
         if ( menu_box_info.ec == 'S' )
           break; /* consider it ok */
+        if ( menu_box_info.ec == 'I' )
+          {
+          ignore_copy_errors = 1;
+          break;
+          }  
         else
           {
           closedir(dir);
@@ -502,7 +513,7 @@ int __vfu_dir_move( const char* src, const char* dst, CopyInfo* copy_info )
   return 0;
 };
 
-/*###########################################################################*/
+/*---------------------------------------------------------------------------*/
 
 int __vfu_link_copy( const char* src, const char* dst, CopyInfo* copy_info )
 {
@@ -545,17 +556,16 @@ int __vfu_link_move( const char* src, const char* dst, CopyInfo* copy_info )
   return 0;
 };
 
-/*###########################################################################*/
+/* erase *******************************************************************/
+
+//  return 0 for ok, CR_ABORT for cancel, else for error
 
-/*
-  return 0 for ok, CR_ABORT for cancel, else for error
-*/
 int __vfu_dir_erase( const char* target, fsize_t* bytes_freed )
 {
   errno = 0; /* clear error status */
   
-  String fname; /* used for directory entries */
-  String s;
+  VString fname; /* used for directory entries */
+  VString s;
   
   if ( vfu_break_op() ) 
     return CR_ABORT; /* canceled */
@@ -585,14 +595,7 @@ int __vfu_dir_erase( const char* target, fsize_t* bytes_freed )
       /* progress report */
       say1( fname );
       
-      int r = 0;
-      if ( file_is_link( fname ) )
-        r = __vfu_link_erase( fname ); /* symlink */
-      else
-      if ( file_is_dir( fname ) )  
-        r = __vfu_dir_erase( fname, bytes_freed ); /* directory */
-      else
-        r = __vfu_file_erase( fname, bytes_freed ); /* regular file */
+      int r = __vfu_erase( fname, bytes_freed );
       
       if ( r == CR_ABORT )
         {
@@ -601,14 +604,22 @@ int __vfu_dir_erase( const char* target, fsize_t* bytes_freed )
         }
       if ( r && r != CR_SKIP )
         {
+        if ( ignore_copy_errors ) break;
+        
         say1( fname );
         say2errno();
-        vfu_menu_box( "Erase error", "T Try again,S Skip/continue,  Abort (ESC)" );
+        vfu_menu_box( "Erase error", 
+                      "T Try again,S Skip/continue,I Ignore further errors,  Abort (ESC)" );
         if ( menu_box_info.ec == 'T' )
           continue; /* while(4) */
         else
         if ( menu_box_info.ec == 'S' )
           break; /* consider it ok */
+        if ( menu_box_info.ec == 'I' )
+          {
+          ignore_copy_errors = 1;
+          break;
+          }
         else
           {
           closedir(dir);
@@ -624,8 +635,8 @@ int __vfu_dir_erase( const char* target, fsize_t* bytes_freed )
   /* show bytes freed if required */
   if ( bytes_freed )
     {
-    String t;
-    t.setfi( *bytes_freed );
+    VString t;
+    t.fi( *bytes_freed );
     str_comma( t );
     t = "ERASE: " + t + " bytes freed.";
     say2(t);
@@ -658,7 +669,7 @@ int __vfu_file_erase( const char* target, fsize_t* bytes_freed  )
 
 /*---------------------------------------------------------------------------*/
 
-int __vfu_link_erase( const char* target )
+int __vfu_link_erase( const char* target, fsize_t* bytes_freed )
 {
   errno = 0; /* clear error status */
   
@@ -669,7 +680,77 @@ int __vfu_link_erase( const char* target )
   #endif
 };
 
-/*###########################################################################*/
+
+/* shells, call __*_*_*() above ********************************************/
+
+int __vfu_copy( const char* src, const char* dst, CopyInfo* copy_info )
+{
+  int r = 0;
+  if ( file_is_link( src ) )
+    r = __vfu_link_copy( src, dst, copy_info ); /* symlink */
+  else
+  if ( file_is_dir( src ) )  
+    r = __vfu_dir_copy( src, dst, copy_info ); /* directory */
+  else
+    r = __vfu_file_copy( src, dst, copy_info ); /* regular file */
+  
+  return r;  
+};
+
+/*---------------------------------------------------------------------------*/
+
+int __vfu_move( const char* src, const char* dst, CopyInfo* copy_info )
+{
+  int r = 0;
+  if ( file_is_link( src ) )
+    r = __vfu_link_move( src, dst, copy_info ); /* symlink */
+  else
+  if ( file_is_dir( src ) )  
+    r = __vfu_dir_move( src, dst, copy_info ); /* directory */
+  else
+    r = __vfu_file_move( src, dst, copy_info ); /* regular file */
+  
+  return r;  
+};
+
+/*---------------------------------------------------------------------------*/
+
+int __vfu_symlink( const char* src, const char* dst, CopyInfo* copy_info )
+{
+  errno = 0; /* clear error status */
+  
+  if (!over_if_exist( src, dst, copy_info )) 
+    return copy_info->abort ? CR_ABORT : CR_SKIP; /* ok */
+  
+  if ( file_exist( dst ) )
+    {
+    if ( file_is_same( src, dst ) == 0 )
+      return CR_SKIP; /* dst is src actually */
+    /* FIXME: what if dst is symlink? */  
+    if ( __vfu_file_erase( dst ) ) return 1;
+    }
+  int r = symlink( src, dst );
+  return ( r != 0 );
+}
+
+/*---------------------------------------------------------------------------*/
+
+int __vfu_erase( const char* target, fsize_t* bytes_freed )
+{
+  int r = 0;
+  
+  if ( file_is_link( target ) )
+    r = __vfu_link_erase( target, bytes_freed ); /* symlink */
+  else
+  if ( file_is_dir( target ) )  
+    r = __vfu_dir_erase( target, bytes_freed ); /* directory */
+  else
+    r = __vfu_file_erase( target, bytes_freed ); /* regular file */
+  
+  return r;  
+};
+
+/* high-level interface functions ******************************************/
 
 void __copy_calc_totals( CopyInfo &copy_info, int a_one )
 {
@@ -688,6 +769,7 @@ void __copy_calc_totals( CopyInfo &copy_info, int a_one )
 
 void vfu_copy_files( int a_one, int a_mode )
 {
+  ignore_copy_errors = 0;
   if ( files_count == 0 )
     {
     say1( "No files" );
@@ -711,9 +793,9 @@ void vfu_copy_files( int a_one, int a_mode )
   if ( opt.copy_calc_totals == 2 ) /* PRELIMINARY copy calc totals */
     __copy_calc_totals( copy_info, a_one );
 
-  String target = opt.last_copy_path[ a_mode ];
+  VString target = opt.last_copy_path[ a_mode ];
   const char* cm_mode_str[] = { "COPY", "MOVE", "LINK" };
-  String str = cm_mode_str[ a_mode ];
+  VString str = cm_mode_str[ a_mode ];
   if ( a_one )
     str = str + " `" + files_list[FLI]->name_ext() + "' to:";
   else
@@ -722,6 +804,22 @@ void vfu_copy_files( int a_one, int a_mode )
   expand_path( target, t );
   /* str_reduce_path( NULL, t ); */
   str_fix_path( t );
+  str_trim_right( t, 1 );
+  if ( file_exists( t ) && !file_is_dir( t ) )
+    {
+    say1( "Target is file, not directory!" );
+    say2( t );
+    
+    vfu_menu_box( "Error prompt", 
+                  "C Continue anyway,E Erase first,  Abort (ESC)", -1 );
+    if ( menu_box_info.ec == 'E' )
+      {
+      unlink( t );
+      menu_box_info.ec = 'C';
+      }
+    if ( menu_box_info.ec != 'C' ) return; /* abort */
+    }
+  str_fix_path( t );
   target = t;
   strcpy( opt.last_copy_path[ a_mode ], target );
 
@@ -743,21 +841,26 @@ void vfu_copy_files( int a_one, int a_mode )
       say1( t );
       say2( target );
       
-      vfu_menu_box( "Error prompt", "C Continue anyway,  Abort (ESC)", -1 );
+      vfu_menu_box( "Error prompt", 
+                    "C Continue anyway,  Abort (ESC)", -1 );
       if ( menu_box_info.ec != 'C' ) return; /* abort */
       }
     } /* free space check */
 
   copy_info.description = "FILE OPERATION: ";
   copy_info.description += cm_mode_str[ a_mode ];
-  
+  copy_info.description += ": ";
+  sprintf( t, "%.0f", copy_info.files_size );
+  str_comma( t );
+  copy_info.description += t;
+  copy_info.description += " bytes.";
     
   ASSERT( !copy_buff );
   copy_buff = new char[1024*1024];
   ASSERT( copy_buff );
   if (copy_buff == NULL)
     {
-    say1( "Copy error: cannot allocate copy buffer :( ..." );
+    say1( "Copy error: cannot allocate copy buffer" );
     return;
     }
   
@@ -775,33 +878,16 @@ void vfu_copy_files( int a_one, int a_mode )
       src -- is full current item path
       dst -- is target + name-ext only!
     */
-    String src  = fi->full_name();
-    String dst  = target;
+    VString src  = fi->full_name();
+    VString dst  = target;
            dst += fi->name_ext();
     
     int r = 0;
-    if ( fi->is_link() )
-      { /* symlink */
-      if ( a_mode == CM_COPY ) r = __vfu_link_copy( src, dst, &copy_info ); else
-      if ( a_mode == CM_MOVE ) r = __vfu_link_move( src, dst, &copy_info ); else
-      if ( a_mode == CM_LINK ) r = __vfu_link_symlink( src, dst, &copy_info ); else
-      ASSERT(!"Bad copy mode");
-      }
-    else
-    if ( fi->is_dir() )   
-      { /* directory */
-      if ( a_mode == CM_COPY ) r = __vfu_dir_copy( src, dst, &copy_info ); else
-      if ( a_mode == CM_MOVE ) r = __vfu_dir_move( src, dst, &copy_info ); else
-      if ( a_mode == CM_LINK ) r = __vfu_dir_symlink( src, dst, &copy_info ); else
-      ASSERT(!"Bad copy mode");
-      }
-    else
-      { /* regular file */
-      if ( a_mode == CM_COPY ) r = __vfu_file_copy( src, dst, &copy_info ); else
-      if ( a_mode == CM_MOVE ) r = __vfu_file_move( src, dst, &copy_info ); else
-      if ( a_mode == CM_LINK ) r = __vfu_file_symlink( src, dst, &copy_info ); else
-      ASSERT(!"Bad copy mode");
-      }  
+    if ( a_mode == CM_COPY ) r = __vfu_copy( src, dst, &copy_info ); else
+    if ( a_mode == CM_MOVE ) r = __vfu_move( src, dst, &copy_info ); else
+    if ( a_mode == CM_LINK ) r = __vfu_symlink( src, dst, &copy_info ); else
+    ASSERT(!"Bad copy mode");
+    
     if ( r == 0 )
       {
       if ( a_mode == CM_MOVE )
@@ -827,14 +913,14 @@ void vfu_copy_files( int a_one, int a_mode )
   do_draw = 2;     
      
   ASSERT( copy_buff );
-  delete copy_buff;
+  delete [] copy_buff;
   copy_buff = NULL;
   
   say1( "" );
   /* show bytes copied */
   if ( copy_info.current_size > 0 )
     { /* i.e. only if there *are* some bytes copied :) */
-    str.setfi( copy_info.current_size );
+    str.fi( copy_info.current_size );
     str_comma( str );
     str = copy_info.description + " DONE: " + str + " bytes copied.";
     }
@@ -844,12 +930,15 @@ void vfu_copy_files( int a_one, int a_mode )
     str += " DONE";
     }
   say2( str );
+
+  ignore_copy_errors = 0;
 };
 
 /*---------------------------------------------------------------------------*/
 
 void vfu_erase_files( int a_one )
 {
+  ignore_copy_errors = 0;
   if ( files_count == 0 )
     {
     say1( "No files" );
@@ -859,11 +948,11 @@ void vfu_erase_files( int a_one )
   fsize_t bytes_freed = 0;
   fsize_t *bytes_freed_ptr = opt.bytes_freed ? &bytes_freed : NULL;
 
-  String str;
+  VString str;
   fsize_t erase_size = vfu_update_sel_size( a_one );
   if ( erase_size != -1 )
     {
-    str.setfi( erase_size );
+    str.fi( erase_size );
     str_comma( str );
     }
   else
@@ -894,31 +983,22 @@ void vfu_erase_files( int a_one )
     if ( a_one && z != FLI ) continue; 
     if ( !a_one && !fi->sel ) continue;
     
-    String target  = fi->full_name();
+    VString target  = fi->full_name();
            
-    int r = 0;
-    if ( fi->is_link() )
-      { /* symlink */
-      r = __vfu_link_erase( target );
-      }
-    else
-    if ( fi->is_dir() )   
-      { /* directory */
-      r = __vfu_dir_erase( target, bytes_freed_ptr );
-      }
-    else
-      { /* regular file */
-      r = __vfu_file_erase( target, bytes_freed_ptr );
-      }
+    int r = __vfu_erase( target, bytes_freed_ptr );
+      
     if ( r == 0 )
       { delete fi; files_list[z] = NULL; }
-    else if ( r != CR_ABORT )
+    else if ( r != CR_ABORT && !ignore_copy_errors )
       {
       say1( target );
       say2errno();
-      vfu_menu_box( "Erase error", "C Continue operation,  Abort (ESC)" );
-      if ( menu_box_info.ec != 'C' )
+      vfu_menu_box( "Erase error", 
+                    "C Continue operation,I Ignore further errors,  Abort (ESC)" );
+      if ( menu_box_info.ec != 'C' && menu_box_info.ec != 'I' )
         r = CR_ABORT;
+      if ( menu_box_info.ec == 'I' )
+        ignore_copy_errors = 1;
       }
     if ( r == CR_ABORT ) break; /* cancel operation */
     } /* for files_list[] */
@@ -931,15 +1011,163 @@ void vfu_erase_files( int a_one )
   /* show bytes freed if required */
   if ( opt.bytes_freed )
     {
-    str.setfi( bytes_freed );
+    str.fi( bytes_freed );
     str_comma( str );
     str = "ERASE DONE: " + str + " bytes freed.";
     say2( str );
     }
   else
     say2( "ERASE DONE" );
+  
+  ignore_copy_errors = 0;
 };
 
+/***************************************************************************
+**
+** CLIPBOARD
+**
+****************************************************************************/
+
+void clipboard_add()
+{
+  if( sel_count == 0 )
+    {
+    say( 1, cINFO, "CLIPBOARD: no files selected, %d files already in clipboard",
+                     clipboard_copy_info.files_count );
+    return;
+    }
+  
+  Clipboard.undef();
+  clipboard_copy_info.reset();
+  
+  VString keep = "1";
+  
+  int z;
+  for ( z = 0; z < files_count; z++ )
+    {
+    TF *fi = files_list[z];
+    if ( !fi->sel ) continue;
+    Clipboard[ fi->full_name() ] = keep;
+    }
+
+  __copy_calc_totals( clipboard_copy_info, 0 );
+  
+  clipboard_copy_info.no_free_check = !opt.copy_free_space_check;
+  clipboard_copy_info.over_mode = OM_ASK; /* 0 */
+  clipboard_copy_info.abort = 0;
+  
+  say( 1, cINFO, "CLIPBOARD: %d files added.", 
+                 clipboard_copy_info.files_count );
+  say2( "" );               
+};
+
+void clipboard_paste( int mode )
+{
+  VArray va = Clipboard.keys();
+
+  ASSERT(    mode == CLIPBOARD_COPY 
+          || mode == CLIPBOARD_MOVE 
+          || mode == CLIPBOARD_SYMLINK );
+
+  ASSERT( !copy_buff );
+  copy_buff = new char[1024*1024];
+  ASSERT( copy_buff );
+  if (copy_buff == NULL)
+    {
+    say1( "Copy error: cannot allocate copy buffer" );
+    return;
+    }
+  
+  va.reset();
+  const char* ps;
+  while( (ps = va.next()) )
+    {
+    VString dst = work_path + str_file_name_ext( ps );
+    
+    int r;
+    if ( mode == CLIPBOARD_SYMLINK )
+      r = __vfu_symlink( ps, dst, &clipboard_copy_info );
+    else if ( mode == CLIPBOARD_MOVE )
+      r = __vfu_move( ps, dst, &clipboard_copy_info );
+    else // if ( mode == CLIPBOARD_COPY )
+      r = __vfu_copy( ps, dst, &clipboard_copy_info );
+    if ( r == 0 ) clipboard_copy_info.ok_count++;
+    if ( r != 0 && r != CR_SKIP && r != CR_ABORT )
+      {
+      say1( dst + r );
+      say2errno();
+      vfu_menu_box( "Copy/Move/Symlink error", "C Continue operation,  Abort (ESC)" );
+      if ( menu_box_info.ec != 'C' ) r = CR_ABORT;
+      }
+    if ( r == CR_ABORT ) break; // cancel operation
+    }
+  
+  ASSERT( copy_buff );
+  delete [] copy_buff;
+  copy_buff = NULL;
+  
+  vfu_rescan_files( 0 );
+  say( 1, cINFO, "CLIPBOARD: %s: %d of %d files processed ok", 
+                 CB_DESC[ mode ],
+                 clipboard_copy_info.ok_count, 
+                 clipboard_copy_info.files_count );
+                 
+  if ( mode == CLIPBOARD_MOVE ) clipboard_clear();
+};
+
+void clipboard_clear()
+{
+  if( clipboard_copy_info.files_count )
+    say( 2, cINFO, "CLIPBOARD: %d files removed", clipboard_copy_info.files_count );
+  else  
+    say( 2, cINFO, "CLIPBOARD: empty" );
+  Clipboard.undef();
+  clipboard_copy_info.reset();
+};
+
+void clipboard_view()
+{
+  mb = Clipboard.keys();
+  if( mb.count() == 0 )
+    {
+    say2( "CLIPBOARD: empty" );
+    return;
+    }
+  vfu_menu_box( 5, 5, "File Clipboard Content" );
+};
+
+void clipboard_menu( int act )
+{
+  if ( act == 0 )
+    {
+    mb.undef();
+    mb.push( "A Add files to the clipboard" );
+    mb.push( "P Copy files here" );
+    mb.push( "O Move files here" );
+    mb.push( "L Symlink files here" );
+    mb.push( "E Clear clipboard" );
+    mb.push( "V View clipboard" );
+    
+    VString fcnt = clipboard_copy_info.files_count;
+    str_comma( fcnt );
+    VString fsize = clipboard_copy_info.files_size;
+    str_comma( fsize );
+    mb.push( "---  " + fcnt + " files, " + fsize + " bytes" );
+    
+    if ( vfu_menu_box( 5, 5, "File Clipboard " + fcnt + " files, " + fsize + " bytes" ) == -1 ) return;
+    act = menu_box_info.ec;
+    }
+  act = toupper( act );
+  switch( act )
+    {
+    case 'A': clipboard_add(); break;
+    case 'P': clipboard_paste( CLIPBOARD_COPY ); break;
+    case 'O': clipboard_paste( CLIPBOARD_MOVE ); break;
+    case 'L': clipboard_paste( CLIPBOARD_SYMLINK ); break;
+    case 'E': clipboard_clear(); break;
+    case 'V': clipboard_view(); break;
+    }
+}
+
 /*---------------------------------------------------------------------------*/
 
-
