@@ -1,10 +1,12 @@
 /*                              
  *
- * (c) Vladi Belperchinov-Shabanski "Cade" 1996-2000
- * http://www.biscom.net/~cade  <cade@biscom.net>  <cade@datamax.bg>
+ * (c) Vladi Belperchinov-Shabanski "Cade" 1996-2003
+ * http://soul.datamax.bg/~cade  <cade@biscom.net>  <cade@datamax.bg>
  *
  * SEE `README',`LICENSE' OR `COPYING' FILE FOR LICENSE AND OTHER DETAILS!
  *
+ * $Id: vfudir.cpp,v 1.23 2003/06/15 08:53:58 cade Exp $
+ *
  */
 
 #include "vfudir.h"
@@ -20,12 +22,12 @@ VArray size_cache;
 /*###########################################################################*/
 
   void __glob_gdn( const char* a_path, const char* a_fnpattern, 
-                   PSZCluster &a_sc ) // glob getdirname
+                   VArray &a_va ) // glob getdirname
   {
-    String pat = a_fnpattern;
+    VString pat = a_fnpattern;
     pat += "*";
     
-    a_sc.freeall();
+    a_va.undef();
     DIR *dir;
     dirent *de; 
     if ( !a_path || a_path[0] == 0 ) 
@@ -40,12 +42,12 @@ VArray size_cache;
              strcmp( de->d_name, ".." ) == 0 ) continue;
         if ( a_fnpattern[0] == 0 || FNMATCH( pat, de->d_name)==0 )
           { 
-          String str = a_path;
+          VString str; // = a_path;
           str += de->d_name;
-          if ( file_is_dir(str) ) 
+          if ( file_is_dir( a_path + str ) ) 
             {
             str += "/";
-            a_sc.add(str);
+            a_va.push(str);
             }
           } 
         } 
@@ -57,7 +59,7 @@ VArray size_cache;
   FIXME: must call sayX() at the end to clear...
 */
 
-int vfu_get_dir_name( const char *prompt, String &target, int should_exist )
+int vfu_get_dir_name( const char *prompt, VString &target, int should_exist )
 { 
   int res = -1;
   /*
@@ -65,8 +67,7 @@ int vfu_get_dir_name( const char *prompt, String &target, int should_exist )
   leaveok(stdscr, FALSE);
   #endif
   */
-  PSZCluster dir_list;
-  dir_list.create( 32, 32 );
+  VArray dir_list;
    
   say1(prompt); 
   say2(""); 
@@ -87,7 +88,7 @@ int vfu_get_dir_name( const char *prompt, String &target, int should_exist )
   while(1) 
     {
     int mx = con_max_x() - 1;
-    String target_out = target;
+    VString target_out = target;
     if ( (pos < page) || (pos+1 > page + mx) || (page > 0 && pos == page) ) 
       page = pos - mx / 2;
     if ( page < 0 ) page = 0;
@@ -108,7 +109,8 @@ int vfu_get_dir_name( const char *prompt, String &target, int should_exist )
     if (ch == '\\') ch = '/'; /* dos hack :)) */
     if ( ch == '/' && str_find( target, '/' ) == -1 && target[0] == '~' )
       {
-      tilde_expand( target );
+      target = tilde_expand( target );
+      str_fix_path( target );
       pos = str_len( target );
       ch = 0;
       }
@@ -132,13 +134,17 @@ int vfu_get_dir_name( const char *prompt, String &target, int should_exist )
     if ( ch == 9 && str_len( target ) > 0)
       { 
       int z; 
-      dir_list.freeall();
-      String dmain; /* main/base path */
-      String dtail; /* item that should be expanded/glob */
+      dir_list.undef();
+      VString dmain; /* main/base path */
+      VString dtail; /* item that should be expanded/glob */
       
+      dmain = str_file_path( target );
+      dtail = str_file_name_ext( target );
+      
+      /*
       int lastslash = str_rfind(target, '/');
       if ( lastslash == -1 ) 
-        { /* not found */
+        {
         dmain = "";
         dtail = target;
         }
@@ -149,7 +155,8 @@ int vfu_get_dir_name( const char *prompt, String &target, int should_exist )
         str_sleft( dmain, lastslash+1 );
         str_trim_left( dtail, lastslash+1 );
         }
-  
+      */
+      
       __glob_gdn( dmain, dtail, dir_list );
   
       z = dir_list.count()-1;
@@ -159,25 +166,28 @@ int vfu_get_dir_name( const char *prompt, String &target, int should_exist )
           {
           int mc = 0; /* match count        */
           int mi = 0; /* match letter index */
-          int li; /* counter */
-          do
+          while(4)
             {
-            mi++;
             mc = 0;
+            int li; /* counter */
             for ( li = 0; li < dir_list.count(); li++ )
+              {
               if ( str_get_ch( dir_list[ 0], mi ) == 
                    str_get_ch( dir_list[li], mi ) )
                 mc++;
+              }
+            if ( mc != dir_list.count() )
+              break;
+            mi++;
             }
-          while( mc == dir_list.count() );
-          target.setn( dir_list[0], mi );
+          target.setn( dmain + dir_list[0], str_len( dmain ) + mi );
           pos = str_len( target );
           say2( target, cINPUT );
           con_xy( pos+1, con_max_y() );
           
           vfu_beep();
           ch = con_getch();
-          if ( ch != 9 ) { dir_list.freeall(); continue; }
+          if ( ch != 9 ) { dir_list.undef(); continue; }
           dir_list.sort();
           con_chide();
           z = vfu_menu_box( 10, 5, "Complete...", &dir_list );
@@ -187,11 +197,15 @@ int vfu_get_dir_name( const char *prompt, String &target, int should_exist )
         else
           ch = 0;
         if ( z != -1 )
-          target = dir_list[z];
+          {
+          while( str_len( target ) > 0 && target[-1] != '/' )
+            str_chop( target );
+          target += dir_list[z];
+          }
         
         pos = str_len( target );
         
-        dir_list.freeall();
+        dir_list.undef();
         if (ch != 0) continue;
         }
       else
@@ -221,13 +235,15 @@ int vfu_get_dir_name( const char *prompt, String &target, int should_exist )
     if (ch == KEY_CTRL_X)
       {
         char t[MAX_PATH];
+        if ( target[0] == '~' )
+          target = tilde_expand( target );
         expand_path( target, t );
         str_fix_path( t );
         target = t;
         pos = str_len( target );
       }
     else 
-    if (ch > 31 && ch < 129 ) // && pos < 70) 
+    if (ch >= 32 && ch <= 255 ) // && pos < 70) 
       { 
       if (firsthit) 
         {
@@ -277,7 +293,10 @@ int vfu_get_dir_name( const char *prompt, String &target, int should_exist )
   //------------------------------------------------------------------
   str_cut_spc( target );
   if ( res == 1 && target[0] == '~' )
-    tilde_expand( target );
+    {
+    target = tilde_expand( target );
+    str_fix_path( target );
+    }
 /*  
   if ( target.len() > 0 )
     { 
@@ -300,7 +319,7 @@ int vfu_get_dir_name( const char *prompt, String &target, int should_exist )
   if ( res == 1 && str_len( target ) > 0 && should_exist && !dir_exist( target ))
     {
     vfu_beep();
-    int ch = tolower( vfu_ask( "Dir NOT exist! Create? "
+    int ch = tolower( vfu_ask( "Directory does not exist! Create? "
                                "( ENTER=Yes, ESC=cancel )",
                                "\033\rcC" ));
     if ( ch == 27 ) 
@@ -332,8 +351,6 @@ int vfu_get_dir_name( const char *prompt, String &target, int should_exist )
     
   str_cut_spc( target );
   
-  dir_list.done();
-  
   ASSERT( res == 0 || res == 1 );
   return res;
 } 
@@ -343,8 +360,8 @@ int vfu_get_dir_name( const char *prompt, String &target, int should_exist )
 void vfu_chdir( const char *a_new_dir )
 {
   char t[MAX_PATH];
-  String target;
-  if (a_new_dir)
+  VString target;
+  if ( a_new_dir && a_new_dir[0] )
     {
     target = a_new_dir;
     str_fix_path( target );
@@ -373,7 +390,7 @@ void vfu_chdir( const char *a_new_dir )
       {
       int z = 0;
       if ( dir_tree.count() == 0 ) tree_load();
-      mb.freeall();
+      mb.undef();
       z = tree_find( target, &mb );
       if (z > 1)
         {
@@ -387,7 +404,7 @@ void vfu_chdir( const char *a_new_dir )
       if (z == 1)
         target = mb.get(0);
       }
-  String str = target; 
+  VString str = target; 
   str_cut_spc( str );
   #ifdef _TARGET_GO32_
     if ( str[0] == '/' )
@@ -411,12 +428,11 @@ void vfu_chdir( const char *a_new_dir )
     {
     str = work_path + str;
     str_fix_path( str );
-    str_reduce_path( str, t );
-    target = t;
+    target = str_reduce_path( str );
     }
   if (chdir( target ) != 0)
     {
-    sprintf( t, "chdir: %s", target.asis() );
+    sprintf( t, "chdir: %s", target.data() );
     say1( t );
     say2errno();
     return;
@@ -428,11 +444,6 @@ void vfu_chdir( const char *a_new_dir )
       work_mode = WM_NORMAL;
     }
   if ( ch != work_path[0] ) tree_drop(); /* drop tree--it is for another drive*/
-  if ( dir_tree.count() == 0 && opt.tree_sizes && opt.tree_auto )
-    {
-    tree_load();
-    if ( dir_tree.count() == 0 ) tree_rebuild();
-    }
   vfu_read_files();
 } 
 
@@ -451,7 +462,7 @@ void vfu_chdir_history()
 
 void tree_load()
 {
-  if (LoadFromFile( filename_tree, &dir_tree, MAX_PATH + 64 ))
+  if( dir_tree.fload( filename_tree ) )
     say1( "DirTree load error." );
   else
     {
@@ -464,7 +475,7 @@ void tree_load()
 
 void tree_save()
 {
-  if (SaveToFile( filename_tree, &dir_tree ))
+  if( dir_tree.fsave( filename_tree ) )
     say1( "DirTree save error." );
   else
     {
@@ -479,7 +490,7 @@ void tree_drop()
 {
   if ( dir_tree_changed )
     tree_save();
-  dir_tree.freeall();
+  dir_tree.undef();
   dir_tree_changed = 0;
 }
 
@@ -490,8 +501,8 @@ void tree_fix()
   int z;
   for( z = dir_tree.count() - 1; z >= 0; z-- )
     {
-    String s1 = dir_tree[z];
-    String s2;
+    VString s1 = dir_tree[z];
+    VString s2;
     if (z < dir_tree.count() - 1)
       s2 = dir_tree[z+1];
     else
@@ -513,7 +524,7 @@ void tree_fix()
       n--;
       }
     if ( p )
-      dir_tree.put( z, s1 );
+      dir_tree.set( z, s1 );
     }
 }
 
@@ -558,7 +569,7 @@ fsize_t __tree_rebuild_process( const char* path )
       int trim = 0;
       for ( z = 0; z < trim_tree.count(); z++ )
         {
-        String trim_temp = trim_tree[z];
+        VString trim_temp = trim_tree[z];
         str_fix_path( trim_temp );
         if ( pathcmp(trim_temp, new_name) == 0 ) 
           { /* trim_tree item found */
@@ -589,9 +600,7 @@ fsize_t __tree_rebuild_process( const char* path )
   closedir(dir);
   
   /* show some progress :) */
-  strcpy( new_name, path );
-  str_dot_reduce( NULL, new_name, con_max_x()-1 );
-  say2( new_name );
+  say2( str_dot_reduce( path, con_max_x()-1 ) );
   return size;
 }
 
@@ -604,8 +613,8 @@ _djstat_flags = _STAT_INODE | _STAT_EXEC_EXT | _STAT_EXEC_MAGIC |
                 _STAT_WRITEBIT;
 // _djstat_flags = 0;
 #endif
-  dir_tree.freeall();
-  size_cache.zap();
+  dir_tree.undef();
+  size_cache.undef();
   say1( "Rebuilding tree..." );
   
   __tree_rebuild_process( "/" );
@@ -616,8 +625,7 @@ _djstat_flags = _STAT_INODE | _STAT_EXEC_EXT | _STAT_EXEC_MAGIC |
 #endif
 
   dir_tree_changed = 1;
-  if (opt.tree_auto) 
-    tree_save();
+  tree_save();
 };
 
 /*-----------------------------------------------------------------------*/
@@ -625,9 +633,9 @@ _djstat_flags = _STAT_INODE | _STAT_EXEC_EXT | _STAT_EXEC_MAGIC |
 void tree_draw_item( int page, int index, int hilite )
 {
   if ( page + index >= dir_tree.count() ) return;
-  String s1 = dir_tree[page+index];
+  VString s1 = dir_tree[page+index];
   str_trim_right(s1,1);
-  String s2 = s1;
+  VString s2 = s1;
   int j = str_rfind( s1,'/');
   str_trim_right(s1,str_len(s2)-j-1);
   str_trim_left(s2,j+1);
@@ -656,6 +664,19 @@ void tree_draw_item( int page, int index, int hilite )
     s1 += "--";
     }
 
+  VString str = dir_tree[page+index];
+  str_tr( str,"\\", "/" );
+  
+  VString sz;
+  sz.fi( size_cache_get( str ) );
+  if ( sz == "-1" )
+    sz = "n/a";
+  else
+    str_comma( sz );
+  str_pad( sz, 14 );
+  
+  s1 = sz + " " + s1;
+
   int m = con_max_x() - 1; /* doesn't speed the code... :) */
   if ( str_len( s1 ) > m )
     {
@@ -684,19 +705,19 @@ void tree_draw_item( int page, int index, int hilite )
 
 /*-----------------------------------------------------------------------*/
 
-void tree_draw_page( TScrollPos &scroll )
+void tree_draw_page( ScrollPos &scroll )
 {
-  String str = " ";
+  VString str = " ";
   str_mul( str, con_max_x() );
-  str = "DiRECTORY" + str;
+  str = "          SiZE DiRECTORY" + str;
   str_sleft( str, con_max_x()-16 ); 
   con_out(1,3, str, cHEADER );
   int z = 0;
-  for(z = 0; z < scroll.pagesize; z++)
+  for(z = 0; z < scroll.pagesize(); z++)
     {
-    if (scroll.page + z <= scroll.max)
+    if (scroll.page() + z <= scroll.max())
       {
-      tree_draw_item( scroll.page, z );
+      tree_draw_item( scroll.page(), z );
       }
     else
       {
@@ -709,36 +730,33 @@ void tree_draw_page( TScrollPos &scroll )
 
 /*-----------------------------------------------------------------------*/
 
-void tree_draw_pos( TScrollPos &scroll, int opos )
+void tree_draw_pos( ScrollPos &scroll, int opos )
 {
-  int z = scroll.pos - scroll.page;
-  if ( opos != -1 ) tree_draw_item( scroll.page, opos );
-  tree_draw_item( scroll.page, z, 1 );
-  String str;
-  str = dir_tree[scroll.pos];
+  int z = scroll.pos() - scroll.page();
+  if ( opos != -1 ) tree_draw_item( scroll.page(), opos );
+  tree_draw_item( scroll.page(), z, 1 );
+  VString str;
+  str = dir_tree[scroll.pos()];
   str_tr( str,"\\", "/" );
   
-  String sz;
-  sz.setfi( size_cache_get( str ) );
+  VString sz;
+  sz.fi( size_cache_get( str ) );
   str_comma( sz );
-  str_pad( sz, 12 );
-  str = sz + "  " + str;
+  str_pad( sz, 14 );
+  str = sz + " " + str;
 
-  str_dot_reduce( NULL, str, con_max_x()-1 );
+  str = str_dot_reduce( str, con_max_x()-1 );
   
-  con_out( 1, con_max_y()-3, str, cINFO ); 
-  con_ce( cINFO );
-  con_out( 1, con_max_y()-2, 
-          "----- R Rebuild --- L Load --- W Write/Save --- S Inc. search --- Z Size -----" , cINFO2 );
-  con_ce( cINFO2 );
-  show_pos( scroll.pos+1, scroll.max+1 );
+  say1( str, cINFO );
+  say2( "         Help: R Rebuild, S Incremental search, Z Recalc directory size", cINFO );
+  show_pos( scroll.pos()+1, scroll.max()+1 );
 }
 
 /*-----------------------------------------------------------------------*/
 
 void tree_view()
 {
-  String str;
+  VString str;
   if (dir_tree.count() == 0)
     {
     tree_load();
@@ -757,15 +775,16 @@ void tree_view()
   set.set_str1( "._-~" );
   set.set_str1( "?*>[]" );
   
-  TScrollPos scroll;
-  scroll.create( 0, dir_tree.count()-1, 0, 0, PS, 1 );
-  scroll.gotopos( new_pos );
-  scroll.type = opt.dynamic_scroll;
+  ScrollPos scroll;
+  scroll.set_min_max( 0, dir_tree.count()-1 );
+  scroll.set_pagesize( PS+2 );
+  scroll.go( new_pos );
   
   int key = 0;
   int opos = -1;
   int opage = -1;
-  while( key != 27 && key != 13 && key != '-' )
+  while( key != 27 && key != 13 && key != '-' && 
+         toupper( key ) != 'Q' && toupper( key ) != 'X' && key != KEY_ALT_X )
     {
     if ( key >= 'A' && key <= 'Z' ) key = tolower( key );
     if ( key == 's' )
@@ -781,37 +800,42 @@ void tree_view()
           if ( key != 9 )
             str_add_ch( str, key );
           say2( str );
+          
+          if ( dir_tree.count() == 0 ) { key = con_getch(); continue; }
+          
           int z;
           if ( key == 9 )
             {
-            z = scroll.pos + 1;
-            if (z > scroll.max ) z = scroll.min;
+            z = scroll.pos() + 1;
+            if (z > scroll.max() ) z = scroll.min();
             }
           else
-            z = scroll.pos;
+            z = scroll.pos();
           int direction = 1;
           int found = 0;
+          int loops = 0;
+          VString s_mask = str;
+          vfu_expand_mask( s_mask );
           while(1)
             {
-            if ( scroll.pos > 0 && z == scroll.pos-1 ) break;
-            if ( scroll.pos == 0 && z == scroll.max ) break;
-            String str1 = dir_tree[z];
+            if ( z > scroll.max() ) z = scroll.min();
+            if ( z < scroll.min() ) z = scroll.max();
+            
+            VString str1 = dir_tree[z];
             str_trim_right( str1, 1 );
             int j = str_rfind(str1,'/');
-            if (j == 0 || j == str_len(str1)-1) continue;
-            str_trim_left( str1, j+1 );
-
-            String s = str;
-            vfu_expand_mask( s );
-            found = ( FNMATCH( s, str1 ) == 0 );
-            if (found) break;
+            if (j < 0) 
+              str1 = "";
+            else
+              str_trim_left( str1, j+1 );
+            found = ( FNMATCH( s_mask, str1 ) == 0 );
+            if ( found ) break;
             z += direction;
-            if ( z > scroll.max ) z = scroll.min;
-            if ( z < scroll.min ) z = scroll.max;
+            if ( loops++ > dir_tree.count() ) break;
             }
           if (found)
             {
-            scroll.gotopos(z);
+            scroll.go(z);
             tree_draw_page( scroll );
             tree_draw_pos( scroll, opos );
             }
@@ -825,25 +849,23 @@ void tree_view()
       {
       case KEY_UP     : scroll.up(); break;
       case KEY_DOWN   : scroll.down(); break;
-      case KEY_PPAGE  : scroll.pageup(); break;
-      case KEY_NPAGE  : scroll.pagedown(); break;
+      case KEY_PPAGE  : scroll.ppage(); break;
+      case KEY_NPAGE  : scroll.npage(); break;
       case KEY_HOME   : scroll.home(); break;
       case KEY_END    : scroll.end(); break;
       case 'r'        : tree_rebuild();
-                        scroll.pos = 0;
-                        scroll.page = 0;
-                        scroll.max = dir_tree.count()-1;
+                        scroll.set_min_max( 0, dir_tree.count()-1 );
+                        scroll.home();
                         say1( "Rebuild done." );
                         break;
       case 'w'        : tree_save(); break;
       case 'l'        : tree_load(); 
-                        scroll.pos = 0;
-                        scroll.page = 0;
-                        scroll.max = dir_tree.count()-1;
+                        scroll.set_min_max( 0, dir_tree.count()-1 );
+                        scroll.home();
                         break;
       case 'z'        : 
       case KEY_CTRL_Z :
-                        str = dir_tree[scroll.pos];
+                        str = dir_tree[scroll.pos()];
                         str_tr( str, "\\", "/" );
                         size_cache_set( str, vfu_dir_size( str ) );
                         tree_draw_page( scroll );
@@ -851,17 +873,17 @@ void tree_view()
                         say1( "Done." );
                         break;
       }
-    if (opage != scroll.page) 
+    if (opage != scroll.page()) 
       tree_draw_page( scroll );
-    if (opos != scroll.pos - scroll.page || opage != scroll.page) 
+    if (opos != scroll.pos() - scroll.page() || opage != scroll.page()) 
       tree_draw_pos( scroll, opos );
-    opos = scroll.pos - scroll.page;
-    opage = scroll.page;
+    opos = scroll.pos() - scroll.page();
+    opage = scroll.page();
     key = con_getch();
     }
   if ( key == 13 )
     {
-    str = dir_tree[scroll.pos];
+    str = dir_tree[scroll.pos()];
     str_tr( str, "\\", "/" );
     vfu_chdir( str );
     }
@@ -869,57 +891,100 @@ void tree_view()
 };
 
 /*###########################################################################*/
-int last_cache_ptr = 0;
+
+#define SIZE_CACHE_OFFSET 12
+
+int size_cache_cmp( const char* s1, const char* s2 )
+{
+  return strcmp( s1 + SIZE_CACHE_OFFSET, s2 + SIZE_CACHE_OFFSET );
+}
+
+VString size_cache_compose_key( const char *s, fsize_t size )
+{
+  const char *ps;
+  char ss[MAX_PATH];
+  if ( opt.fast_size_cache )
+    {
+    ps = s;
+    }
+  else
+    {  
+    expand_path( s, ss );
+    ps = ss;
+    }
+  
+  char s_adler[16];
+  char s_size[32];
+  
+  sprintf( s_size, "%012.0f", size ); //WARNING: THIS IS SIZE_CACHE_OFFSET
+  sprintf( s_adler, "%08X", (unsigned int)str_adler32( ps ) );
+  
+  VString str;
+  str = str + s_size + "|" + s_adler + "|" + ps;
+  
+  return str;
+}
 
 int size_cache_index( const char *s )
 {
   if ( size_cache.count() == 0 ) return -1;
-  int z = last_cache_ptr;
-  if ( z >= size_cache.count() ) z = 0; // just to be safe
-  while(42)
+  VString str = size_cache_compose_key( s, 0 );
+  int l = 0;
+  int h = size_cache.count() - 1;
+  int m = h;
+  while(4)
     {
-    if ( strcmp( size_cache.get2( z ), s ) == 0 ) 
-      {
-      last_cache_ptr = z;
-      return z;
-      }
-    z += 2;
-    if ( z >= size_cache.count() ) z = 0;
-    if ( z == last_cache_ptr ) break;
+    int r =  size_cache_cmp( size_cache[m], str );
+    if ( l == m && r != 0 )
+      return -1;
+    if ( r > 0 )
+      h = m; else
+    if ( r < 0 )
+      l = m; 
+    else
+      return m;
+    m = ( l + h ) / 2;
     }
-  return -1;  
-};
+}
 
 fsize_t size_cache_get( const char *s )
 {
-  char t[16];
-  sprintf( t, "%X", str_adler32( s ) );
-  String str = t;
-  str += " ";
-  str += s;
-  int z = size_cache_index( str );
-  if ( z == -1 )
-    return -1;
+  int z = size_cache_index( s );
+  if ( z != -1 )
+    {
+    VString str = size_cache[z];
+    str_sleft( str, SIZE_CACHE_OFFSET );
+    return atof( str );
+    }
   else
-    return atof( size_cache.get( z+1 ) );  
+    return -1;  
 }
 
 void size_cache_set( const char *s, fsize_t size )
 {
-  char t[16];
-  sprintf( t, "%X", str_adler32( s ) );
-  String str = t;
-  str += " ";
-  str += s;
-  int z = size_cache_index( str );
+  int z = size_cache_index( s );
   if ( z != -1 )
+    size_cache.del( z );
+  size_cache.push( size_cache_compose_key( s, size ) );
+  size_cache.sort( 0, size_cache_cmp );
+}
+
+void size_cache_clean( const char *s )
+{
+  VString str = size_cache_compose_key( s, 0 );
+  str_trim_left( str, SIZE_CACHE_OFFSET );
+  int sl = str_len( str );
+  int z = 0;
+  while( z < size_cache.count() )
     {
-    size_cache.del( z );
-    size_cache.del( z );
+    const char* ps = size_cache[z].data() + SIZE_CACHE_OFFSET;
+    if ( ( strncmp( ps, str, sl ) == 0 && (ps[sl] == '/' || ps[sl] == 0) )
+         ||
+         ( size_cache[z][SIZE_CACHE_OFFSET] != '|' ) )
+      size_cache.del( z );
+    else
+      z++;
     }
-  size_cache.add( str );
-  str.setfi( size );
-  size_cache.add( str );
 }
 
 /*###########################################################################*/
@@ -942,10 +1007,13 @@ int tree_index( const char *s )
   ;
   sl1 = strlen( s1 );
 
-  z = __tree_index_last_cache;
+  z = __tree_index_last_cache + 1;
   while(4)
     {
-    char *s2 = dir_tree[z];
+    if ( z >= dir_tree.count() ) z = 0;
+    if ( z == __tree_index_last_cache ) break;
+    
+    const char *s2 = dir_tree[z];
     sl2 = strlen( s2 );
 
     if ( sl1 == sl2 )
@@ -961,10 +1029,6 @@ int tree_index( const char *s )
     ASSERT( z < dir_tree.count() );
     ASSERT( z >= 0 );
     z++;
-    if ( z == dir_tree.count() )
-      z = 0;
-    if ( z == __tree_index_last_cache )
-      break;
     }
   return -1;
 }
@@ -973,7 +1037,7 @@ int tree_index( const char *s )
 
 const char* tree_find( const char *s ) // return full path by dirname
 {
-  String str;
+  VString str;
   int z = 0;
   int sl = strlen( s );
   for ( z = 0; z < dir_tree.count(); z++ )
@@ -990,11 +1054,11 @@ const char* tree_find( const char *s ) // return full path by dirname
 }
 
 /*-----------------------------------------------------------------------*/
-/* return count of found dirnames and stores them to sc */
+/* return count of found dirnames and stores them to va */
 
-int tree_find( const char *s, PSZCluster *sc ) 
+int tree_find( const char *s, VArray *va ) 
 {
-  String str;
+  VString str;
   int z = 0;
   int sl = strlen( s );
   for ( z = 0; z < dir_tree.count(); z++ )
@@ -1006,10 +1070,10 @@ int tree_find( const char *s, PSZCluster *sc )
       {
       str = dir_tree[z];
       str_tr( str, "\\", "/" );
-      sc->add( str );
+      va->push( str );
       }
     }
-  return sc->count();
+  return va->count();
 }
 
 /*###########################################################################*/
@@ -1066,9 +1130,7 @@ fsize_t __dir_size_process( const char* path )
   closedir(dir);
     
   /* show some progress :) */
-  strcpy( new_name, path );
-  str_dot_reduce( NULL, new_name, con_max_x()-1 );
-  say2( new_name );
+  say2( str_dot_reduce( path, con_max_x()-1 ) );
   return size;
 }
 
@@ -1076,11 +1138,10 @@ fsize_t vfu_dir_size( const char *s )
 {
   char t[MAX_PATH];
   expand_path( s, t );
+  size_cache_clean( t );
   str_fix_path( t );
-  
   fsize_t size = __dir_size_process( t );
   size_cache_set( t, size );
-  
   return size;
 }
 
