@@ -1,10 +1,12 @@
 /*
  *
- * (c) Vladi Belperchinov-Shabanski "Cade" 1996-2000
- * http://www.biscom.net/~cade  <cade@biscom.net>  <cade@datamax.bg>
+ * (c) Vladi Belperchinov-Shabanski "Cade" 1996-2003
+ * http://soul.datamax.bg/~cade  <cade@biscom.net>  <cade@datamax.bg>
  *
  * SEE `README',`LICENSE' OR `COPYING' FILE FOR LICENSE AND OTHER DETAILS!
  *
+ * $Id: vfudir.h,v 1.9 2003/01/30 00:03:51 cade Exp $
+ *
  */
 
 #ifndef _VFUDIR_H_
@@ -16,7 +18,7 @@ extern VArray size_cache;
 
 /*###########################################################################*/
 
-int vfu_get_dir_name( const char *prompt, String &target, int should_exist = 1 );
+int vfu_get_dir_name( const char *prompt, VString &target, int should_exist = 1 );
 void vfu_chdir( const char *a_new_dir );
 void vfu_chdir_history();
 
@@ -30,8 +32,8 @@ void tree_rebuild();
 void tree_fix();
 
 void tree_draw_item( int page, int index, int hilite = 0 );
-void tree_draw_page( TScrollPos &scroll );
-void tree_draw_pos( TScrollPos &scroll, int opos );
+void tree_draw_page( ScrollPos &scroll );
+void tree_draw_pos( ScrollPos &scroll, int opos );
 
 /*###########################################################################*/
 
@@ -39,11 +41,13 @@ int  tree_index( const char *s );
 const char* tree_find( const char *s ); /* return full path by dirname */
 
 /* return count of found dirnames and stores them to sc */
-int tree_find( const char *s, PSZCluster *sc ); 
+int tree_find( const char *s, VArray *va ); 
 
+VString size_cache_compose_key( const char *s, fsize_t size );
 int size_cache_index( const char *s );
 fsize_t size_cache_get( const char *s );
 void size_cache_set( const char *s, fsize_t size );
+void size_cache_clean( const char *s );
 
 /*###########################################################################*/
 
