@@ -1,10 +1,12 @@
 /*
  *
- * (c) Vladi Belperchinov-Shabanski "Cade" 1996-2000
- * http://www.biscom.net/~cade  <cade@biscom.net>  <cade@datamax.bg>
+ * (c) Vladi Belperchinov-Shabanski "Cade" 1996-2003
+ * http://soul.datamax.bg/~cade  <cade@biscom.net>  <cade@datamax.bg>
  *
  * SEE `README',`LICENSE' OR `COPYING' FILE FOR LICENSE AND OTHER DETAILS!
  *
+ * $Id: vfufiles.cpp,v 1.14 2003/04/28 17:17:01 cade Exp $
+ *
  */
 
 #include "vfu.h"
@@ -47,37 +49,36 @@ void vfu_rescan_files( int a_recursive )
 {
   int z;
   int old_fli = FLI;
+  int old_flp = FLP;
+
+  VString keep = "1";
 
   /* save selection, remember which files are selected */
-  String saves = "";
+  VTrie savea;
+  int savea_count = 0;
   if ( opt.keep_selection && sel_count > 0 )
     {
-    saves += ":";
     for ( z = 0; z < files_count ; z++)
       if ( files_list[z]->sel )
         {
-        saves += files_list[z]->name();
-        saves += ":";
+        savea[ files_list[z]->name() ] = keep;
+        savea_count++;
         }
     }
 
   vfu_read_files( a_recursive );
 
   /* restore selection */
-  if ( opt.keep_selection && str_len( saves ) )
+  if ( opt.keep_selection && savea_count > 0 )
     {
     for ( z = 0; z < files_count ; z++ )
-      {
-      String tmp;
-      tmp  = ":";
-      tmp += files_list[z]->name();
-      tmp += ":";
-      files_list[z]->sel = ( str_find( saves, tmp ) != -1 );
-      }
+      if ( savea.exists( files_list[z]->name() ) )
+        files_list[z]->sel = 1;
     update_status();
     }
 
-  FGO(old_fli);
+  file_list_index.set_page( old_flp );
+  file_list_index.set_pos( old_fli );
   vfu_nav_update_pos();
 };
 
@@ -92,19 +93,14 @@ void vfu_read_files( int a_recursive )
   /* clear files list -- delete all found entries */
   for ( z = 0; z < MAX_FILES ; z++) 
     if (files_list[z]) 
-      { delete files_list[z]; files_list[z] = NULL; }
+      { 
+      delete files_list[z]; 
+      files_list[z] = NULL; 
+      }
 
   /* vfu_add_file() will need this */
   files_count = 0;
 
-  /* load/rebuild tree if tree_sizes is on */
-  if ( opt.tree_sizes && dir_tree.count() == 0 )
-    {
-    tree_load();
-    if ( dir_tree.count() == 0 && opt.tree_auto )
-      tree_rebuild();
-    }
-
   /* FIXME: perhaps we could check work_mode here? ... anyway will ASSERT it */
   if ( archive_name != "" )
     {
@@ -128,10 +124,8 @@ void vfu_read_files( int a_recursive )
     }
 
   /* update scroll parameters */
-  file_list_index.min = 0;
-  file_list_index.max = files_count - 1;
-  file_list_index.page = 0;
-  file_list_index.pagesize = con_max_y() - 7;
+  file_list_index.set_min_max( 0, files_count - 1 );
+  file_list_index.set_pagesize( con_max_y() - 7 );
 
   update_status();
   vfu_nav_update_pos();
@@ -150,10 +144,9 @@ void vfu_read_files( int a_recursive )
 int vfu_add_file( const char* fname, const struct stat *st, int is_link )
 {
   if ( files_count == MAX_FILES ) return 1;
-  char t[MAX_PATH];
-  str_file_name_ext( fname, t );
+  VString ne = str_file_name_ext( fname );
 
-  if ( strcmp( t, "." ) == 0 || strcmp( t, ".." ) == 0 ) return 0;
+  if ( ne == "."  || ne == ".." ) return 0;
   
   /* now try to hide `system/special' files */
   if ( !opt.show_hidden_files )
@@ -163,25 +156,33 @@ int vfu_add_file( const char* fname, const struct stat *st, int is_link )
       file_get_mode_str( st->st_mode, mode_str );
       if ( mode_str[7] == 'H' || mode_str[8] == 'S' ) return 0;
     #else
-      if ( t[0] == '.' ) return 0;
+      if ( ne[0] == '.' ) return 0;
     #endif  
     }
 
   if ( !S_ISDIR( st->st_mode ) ) /* mask is not allowed for dirs */
-    if ( vfu_fmask_match( t ) ) return 0; /* doesn't match the mask */
+    if ( vfu_fmask_match( ne ) ) return 0; /* doesn't match the mask */
   TF *fi = new TF( fname, st, is_link );
   files_list[files_count] = fi;
   files_count++;
 
-  /* get dir sizes from tree */
-  if ( opt.tree_sizes && work_mode == WM_NORMAL && fi->is_dir() )
+  /* get dir sizes for directories, not symlinks */
+  if ( work_mode == WM_NORMAL && fi->is_dir() && !fi->is_link() )
     fi->set_size( size_cache_get( fi->full_name( 1 ) ) );
+  /* get dir sizes for directories, symlinks */
+  if ( work_mode == WM_NORMAL && fi->is_dir() && fi->is_link() )
+    {
+    char t[MAX_PATH];
+    expand_path( fi->full_name( 1 ), t );
+    str_fix_path( t );
+    fi->set_size( size_cache_get( t ) );
+    }
 
   /* show progress ... */
   if ( files_count % 123 == 0 )
     {
-    sprintf(t, "Rescanning files... (%5d)  ", files_count);
-    say1( t );
+    sprintf( ne, "Rescanning files... (%5d)  ", files_count);
+    say1( ne );
     }
   return 0;
 };
@@ -194,7 +195,7 @@ int __vfu_ftw_add( const char* origin, const char* fname,
   if ( vfu_break_op() ) return 1;
   if ( flag == FTWALK_DX ) return 0; /* exit directory */
 
-  String str = fname;
+  VString str = fname;
   str_trim_left( str, str_len( origin ) );
   
   return vfu_add_file( str, st, file_is_link( fname ) );
@@ -204,14 +205,16 @@ void vfu_read_local_files( int a_recursive )
 {
   ftwalk( ".", __vfu_ftw_add, a_recursive ? -1 : 1 );
   
-  if ( opt.auto_mount && files_count == 1 && FNMATCH( files_list[0]->name(), "automount" ) == 0 )
+  if ( opt.auto_mount && files_count == 1 && 
+       ( FNMATCH( files_list[0]->name(), "automount" ) == 0 ||
+         FNMATCH( files_list[0]->name(), ".automount" ) == 0 ) )
    {
-   String tmp_file_name;
+   VString tmp_file_name;
    tmp_file_name += tmp_path;
    tmp_file_name += "vfu_automount_error.";
    tmp_file_name += user_id_str;
    
-   String str = work_path;
+   VString str = work_path;
    chdir( "/" );
    str = "mount " + str + " 2> " + tmp_file_name;
    say1( "AutoMount point detected, executing:" );
@@ -271,6 +274,8 @@ void vfu_read_external_files()
       struct stat st;
       stat( tmp1, &st );
       
+      say2( tmp1 );
+      
       if ( vfu_add_file( tmp1, &st, file_is_link( tmp1 ) ) ) 
         {
         pclose(f);
@@ -295,7 +300,7 @@ void vfu_read_pszlist_files()
     stat( pc, &st );
     vfu_add_file( pc, &st, file_is_link( pc )  );
     }
-  list_panelizer.freeall(); /* reset -- there's no reload on this */
+  list_panelizer.undef(); /* reset -- there's no reload on this */
 }
 
 /*---------------------------------------------------------------------------*/
@@ -336,8 +341,8 @@ void vfu_pack_files_list()
   while ( files_count < MAX_FILES && files_list[files_count] ) files_count++;
 
   /* update scroll parameters */
-  file_list_index.max = files_count - 1;
-  file_list_index.pagesize = con_max_y() - 7;
+  file_list_index.set_min_max( 0, files_count - 1 );
+  file_list_index.set_pagesize(  con_max_y() - 7 );
 
   update_status();
   vfu_nav_update_pos();
@@ -349,23 +354,22 @@ void vfu_pack_files_list()
 /* this compares Name20 and Name3 and returns second as smaller :) (or so) */
 int namenumcmp( const char* s1, const char* s2 )
 {
-  int s1l = strlen( s1 );
-  int s2l = strlen( s2 );
-  int z;
-  for ( z = 0; z < ( s1l < s2l ? s1l : s2l ); z++ )
-    if ( s1[z] != s2[z] ) break;
-  if ( z == s1l && z == s2l ) return 0;
-  if ( z == s1l ) return -1;
-  if ( z == s2l ) return  1;
-  String str1 = s1+z;
-  String str2 = s2+z;
-  z = str_find( str1, '.' ); if (z != -1) str_sleft( str1, z );
-  z = str_find( str2, '.' ); if (z != -1) str_sleft( str2, z );
-  if ( str_count( str1, "0123456789" ) == str_len(str1) && 
-       str_count( str2, "0123456789" ) == str_len(str2) )
-     return atoi(str1.asis()) - atoi(str2.asis());
-   else
-     return pathcmp( s1, s2 );
+  VRegexp re1( "^(.*)([0123456789]+)(\\.(.*))?$" );
+  VRegexp re2( "^(.*)([0123456789]+)(\\.(.*))?$" );
+  if ( re1.m(s1) && re2.m(s2) )
+    {
+    VString ss1;
+    VString ss2;
+    sprintf( ss1, "%020d", atoi(re1[2]) );
+    sprintf( ss2, "%020d", atoi(re1[2]) );
+    ss1 = re1[1] + ss1 + re1[3];
+    ss2 = re2[1] + ss2 + re2[3];
+    return pathcmp( ss1, ss2 );
+    }
+  else
+    {
+    return pathcmp( s1, s2 );
+    }  
 }
 
 /*---------------------------------------------------------------------------*/
@@ -396,10 +400,13 @@ switch (opt.sort_order)
  case 'S' : z = (f2->size()  < f1->size()) - (f2->size()  > f1->size());
             break;
 
- case 'D' :
- case 'T' : if ( vfu_opt_time(f1->st()) > vfu_opt_time(f2->st()) ) z = 1; 
-            else
-            if ( vfu_opt_time(f1->st()) < vfu_opt_time(f2->st()) ) z = -1; 
+ case 'T' : z = f1->st()->st_mtime - f2->st()->st_mtime;
+            break;
+
+ case 'H' : z = f1->st()->st_ctime - f2->st()->st_ctime;
+            break;
+
+ case 'C' : z = f1->st()->st_atime - f2->st()->st_atime;
             break;
 
  case 'A' : z = strcmp( f1->mode_str(), f2->mode_str() );
@@ -419,7 +426,7 @@ switch (opt.sort_order)
                 (f2->st()->st_uid  < f1->st()->st_uid);
             break;
 
- case 'Y' : z = 0;
+ case 'Y' : z = strcmp( f1->type_str(), f2->type_str() );
             break;
  default  : ASSERT( !"Non valid sort order (opt.sort_order)" ); break;
  }
@@ -472,7 +479,7 @@ void __vfu_sort(int l, int r)
 void vfu_sort_files()
 {
   if (!files_count) return;
-  String str = files_list[FLI]->name();
+  VString str = files_list[FLI]->name();
   __vfu_sort( 0, files_count - 1 );
   do_draw = 1;
   if ( str != "" )
@@ -493,31 +500,54 @@ void vfu_arrange_files()
 {
   int _ord;
   int _rev;
-  mb.freeall();
-  mb.add( "N Name" );
-  mb.add( "M Name### (RTFM)" );
-  mb.add( "E Extension" );
-  mb.add( "S Size" );
-  mb.add( "T Time" );
-  mb.add( "D Time" );
-  mb.add( "A Attr/mode" );
-  mb.add( "O Owner" );
-  mb.add( "G Group" );
-  mb.add( "U Unsorted" );
-  mb.add( "---" );
-  mb.add( "V Move Entry" );
+  mb.undef();
+  mb.push( "N Name" );
+  mb.push( "M Name### (RTFM)" );
+  mb.push( "E Extension" );
+  mb.push( "S Size" );
+  mb.push( "T Modify Time" );
+  mb.push( "H Change Time" );
+  mb.push( "C Access Time" );
+  mb.push( "A Attr/mode" );
+  mb.push( "O Owner" );
+  mb.push( "G Group" );
+  mb.push( "Y Type (TP)" );
+  mb.push( "U Unsorted" );
+  mb.push( "---" );
+  mb.push( "R Randomize" );
+  mb.push( "V Move Entry" );
+  mb.push( "---" );
+  mb.push( "D Modify Time (compat)" );
   if ( vfu_menu_box( 50, 5, "Arrange" ) == -1 ) return;
   _ord = menu_box_info.ec;
+  if ( _ord == 'D' ) _ord = 'T';
 
   if (_ord == 'V' )
     {
     vfu_file_entry_move();
     return;
     }
+
+  if (_ord == 'R')
+    {
+    /* Fisher-Yates shuffle */
+    int i = files_count - 1;
+    while( i >= 0 )
+      {
+      int j = rand() % ( i + 1 );
+      TF *tmp;
+      tmp = files_list[i];
+      files_list[i] = files_list[j];
+      files_list[j] = tmp;
+      i--;
+      }
+    do_draw = 2;  
+    return;
+    }
   
-  mb.freeall();
-  mb.add( "A Ascending");
-  mb.add( "D Descending" );
+  mb.undef();
+  mb.push( "A Ascending");
+  mb.push( "D Descending" );
   if ( vfu_menu_box( 50, 5, "Order" ) == -1 ) return;
   _rev = menu_box_info.ec;
 
