@@ -1,10 +1,12 @@
 /*
  *
- * (c) Vladi Belperchinov-Shabanski "Cade" 1996-2000
- * http://www.biscom.net/~cade  <cade@biscom.net>  <cade@datamax.bg>
+ * (c) Vladi Belperchinov-Shabanski "Cade" 1996-2003
+ * http://soul.datamax.bg/~cade  <cade@biscom.net>  <cade@datamax.bg>
  *
  * SEE `README',`LICENSE' OR `COPYING' FILE FOR LICENSE AND OTHER DETAILS!
  *
+ * $Id: vfuopt.cpp,v 1.14 2003/04/28 17:17:01 cade Exp $
+ *
  */
 
 #include "vfu.h"
@@ -40,17 +42,16 @@ ToggleEntry Toggles[] =
   { ' ', "TagMark type   ", &(opt.tag_mark_type), TAGMARKS },
   { ' ', "Use colors     ", &(opt.use_colors), NOYES },
   { ' ', "Use /etc/DIR_COLORS", &(opt.use_dir_colors), NOYES },
+  { ' ', "Lowercase extensions for configs", &(opt.lower_case_ext_config), NOYES },
   { '.', "Show hidden files", &(opt.show_hidden_files), NOYES },
   {  0 , "--navigation--", NULL, NULL },
   { 'i', "Use internal viewer", &(opt.internal_browser), NOYES },
   { 'I', "Use internal editor", &(opt.internal_editor), NOYES },
-  { 'y', "Dynamic scroll ", &(opt.dynamic_scroll), NOYES },
   { ' ', "Use menu borders", &(opt.menu_borders), NOYES },
   {  0 , "--trees/dirs-- " , NULL, NULL },
-  { ' ', "Auto DirTree   "   , &(opt.tree_auto), NOYES },
   { ' ', "Compact DirTree", &(opt.tree_compact), NOYES },
-  { ' ', "DirTree Sizes  "  , &(opt.tree_sizes), NOYES },
   { ' ', "CDTree (cdpath) ", &(opt.tree_cd), NOYES },
+  { ' ', "Fast dir size cache ", &(opt.fast_size_cache), NOYES },
   {  0 , "--troubleshooting--", NULL, NULL },
   { ' ', "Clear screen on shell", &(opt.shell_cls), NOYES },
   {  0 , "--compatibility--", NULL, NULL },
@@ -98,18 +99,14 @@ void vfu_load_dir_colors()
 {
   #ifdef _TARGET_UNIX_
 
-  int z;
-  for ( z = 0; z < 16; z++ ) ext_colors[z] = "";
-  
-  PSZCluster sc;
-  sc.create( 16, 16 );
-  LoadFromFile( "/etc/DIR_COLORS", &sc, 1024 );
-  if (sc.count() == 0) return;
+  VArray va;
+  va.fload( "/etc/DIR_COLORS" );
+  if (va.count() == 0) return;
 
-  while( sc.count() )
+  while( va.count() )
     {
-    String str = sc[0];
-    sc.free( 0 );
+    VString str = va[0];
+    va.del( 0 );
     int comment = str_find( str, '#' );
     if ( comment != -1 ) str_sleft( str, comment );
     str_cut( str, " \t" );
@@ -147,86 +144,57 @@ void vfu_load_dir_colors()
 
   for ( int z = 0; z < 16; z++ )
     if( str_len( ext_colors[z] ) > 0 )
+      {
       ext_colors[z] += ".";
+      if ( opt.lower_case_ext_config )
+        str_low( ext_colors[z] );
+      }
     
   #endif /* _TARGET_UNIX_ */
 };
 
 /*---------------------------------------------------------------------------*/
 
-int set_set( const char *line, const char *keyword, char *target )
+int set_arr( const char *line, const char *keyword, VArray &target )
 {
-  char temp[255];
-  regexp *re = regcomp("^[ \011]*([a-zA-Z0-9]+)[ \011]*=[ \011]*(.+)");
-  int r = (regexec(re, line) && !strcasecmp(keyword, regsubn(re, 1, temp)));
-  if (r) regsubn(re, 2, target);
-  free(re);
-  return r;
+  VRegexp re("^[ \011]*([a-zA-Z0-9]+)[ \011]*=[ \011]*(.+)");
+  if ( ! re.m( line ) ) return 0;
+  if ( str_low( re[1] ) != keyword ) return 0;
+  target.push( re[2] );
+  return 1;
 }
 
 /*---------------------------------------------------------------------------*/
 
-int set_set( const char *line, const char *keyword, PSZCluster *target )
+int set_str( const char *line, const char *keyword, VString &target )
 {
-  char temp[255];
-  regexp *re = regcomp("^[ \011]*([a-zA-Z0-9]+)[ \011]*=[ \011]*(.+)");
-  int r = (regexec(re, line) && !strcasecmp(keyword, regsubn(re, 1, temp)));
-  if (r)
-    {
-    regsubn(re, 2, temp);
-    target->add( temp );
-    }
-  free(re);
-  return r;
+  VRegexp re("^[ \011]*([a-zA-Z0-9]+)[ \011]*=[ \011]*(.+)");
+  if ( ! re.m( line ) ) return 0;
+  if ( str_low( re[1] ) != keyword ) return 0;
+  target = re[2];
+  return 1;
 }
 
 /*---------------------------------------------------------------------------*/
 
-int set_set( const char *line, const char *keyword, String &target )
+int set_int( const char *line, const char *keyword, int &target )
 {
-  char temp[255];
-  regexp *re = regcomp("^[ \011]*([a-zA-Z0-9]+)[ \011]*=[ \011]*(.+)");
-  int r = (regexec(re, line) && !strcasecmp(keyword, regsubn(re, 1, temp)));
-  if (r)
-    {
-    regsubn(re, 2, temp);
-    target = temp;
-    }
-  free(re);
-  return r;
+  VRegexp re("^[ \011]*([a-zA-Z0-9]+)[ \011]*=[ \011]*([0123456789]+)");
+  if ( ! re.m( line ) ) return 0;
+  if ( str_low( re[1] ) != keyword ) return 0;
+  target = atoi( re[2] );
+  return 1;
 }
 
 /*---------------------------------------------------------------------------*/
 
-int set_set( const char *line, const char *keyword, int &target )
+int set_splitter( const char *line, const char *keyword, VArray &splitter )
 {
-  char temp[255];
-  regexp *re = regcomp("^[ \011]*([a-zA-Z0-9]+)[ \011]*=[ \011]*([0123456789]+)");
-  int r = (regexec(re, line) && !strcasecmp(keyword, regsubn(re, 1, temp)));
-  if (r)
-    {
-    regsubn(re, 2, temp);
-    target = atoi(temp);
-    }
-  free(re);
-  return r;
-}
-
-/*---------------------------------------------------------------------------*/
-
-int set_set( const char *line, const char *keyword, StrSplitter &splitter )
-{
-  char temp[1024];
-  regexp *re = regcomp("^[ \011]*([a-zA-Z0-9]+)[ \011]*=[ \011]*(.+)");
-  int r = (regexec(re, line) && !strcasecmp(keyword, regsubn(re, 1, temp)));
-  if (r)
-    {
-    regsubn(re, 2, temp);
-    splitter.set_delimiter( PATH_DELIMITER );
-    splitter.set( temp );
-    }
-  free(re);
-  return r;
+  VRegexp re("^[ \011]*([a-zA-Z0-9]+)[ \011]*=[ \011]*(.+)");
+  if ( ! re.m( line ) ) return 0;
+  if ( str_low( re[1] ) != keyword ) return 0;
+  splitter = str_split( PATH_DELIMITER, re[2] );
+  return 1;
 }
 
 /*---------------------------------------------------------------------------*/
@@ -241,12 +209,10 @@ int key_by_name( const char* key_name )
   /*
   if (strcmp (key_name, "MENU"  ) == 0) ux.key = - menucount;
   */
-  regexp *reFKEYS = regcomp( "[\\@\\^\\#]?[fF][01234567890]+" );
-  if (!reFKEYS) return 0;
+  VRegexp reFKEYS( "[\\@\\^\\#]?[fF][01234567890]+" );
   
-  if (regexec( reFKEYS, key_name ))
+  if ( reFKEYS.m( key_name ) )
     {
-    free(reFKEYS); /* so I can return below */
     if ( toupper(key_name[0]) == 'F' )
       return KEY_F1 + atoi( key_name + 1 ) - 1; else
     if ( toupper(key_name[0]) == '@' )
@@ -256,8 +222,6 @@ int key_by_name( const char* key_name )
     if ( toupper(key_name[0]) == '#' )
       return KEY_SH_F1 + atoi( key_name + 2 ) - 1;
     }
-  else  
-    free(reFKEYS);  
   return 0;    
 };
 
@@ -265,14 +229,13 @@ int key_by_name( const char* key_name )
 
 void vfu_settings_load()
 {
-  String str;
-  char t[1024];
+  VString str;
   
-  user_externals.freeall();
-  history.freeall();
-  see_filters.freeall();
-  panelizers.freeall();
-  archive_extensions.freeall();
+  user_externals.undef();
+  history.undef();
+  see_filters.undef();
+  panelizers.undef();
+  archive_extensions.undef();
   
   /***** LOAD DEFAULTS *******/
 
@@ -280,6 +243,7 @@ void vfu_settings_load()
   
   opt.svo.reset();
   opt.seo.reset();
+  opt.seo.handle_tab = 1;
   
   opt.sort_order = 'N'; 
   opt.sort_direction = 'A'; 
@@ -294,10 +258,9 @@ void vfu_settings_load()
   opt.f_time_type = 1;
 
   opt.long_name_view = 0;
-  opt.tree_auto = 0;
   opt.tree_compact = 0;
-  opt.tree_sizes = 1;
   opt.tree_cd = 1;
+  opt.fast_size_cache = 0;
 
   opt.show_hidden_files = 1;
   
@@ -305,6 +268,7 @@ void vfu_settings_load()
 
   opt.use_colors = 1;
   opt.use_dir_colors = 1;
+  opt.lower_case_ext_config = 1;
 
   opt.copy_free_space_check = 1;
   opt.copy_calc_totals = 1;
@@ -324,7 +288,6 @@ void vfu_settings_load()
   opt.menu_borders = 0;
 
   opt.lynx_navigation = 0;
-  opt.dynamic_scroll = 1;
 
   opt.auto_mount = 1;
   opt.keep_selection = 1;
@@ -342,7 +305,7 @@ void vfu_settings_load()
   else
     say1( "warning: bad vfu.options file, loading defaults..." );
   
-  LoadFromFile( filename_history, &history, MAX_PATH );
+  history.fload( filename_history );
   
   if (getenv("EDITOR"))
     {
@@ -364,17 +327,11 @@ void vfu_settings_load()
     shell_browser  = getenv("VIEWER");
     shell_browser  += " %f";
     }
-  
-  regexp *re_ux  = regcomp("^[ \011]*u?x[ \011]*=[ \011]*([^,]*)[ \011]*,[ \011]*([^, \011]*)[ \011]*,[ \011]*([^, \011]*)[ \011]*,(.*)$");
-  regexp *re_see = regcomp( "^[ \011]*see[ \011]*=[ \011]*([^, \011]*)[ \011]*,(.*)$" );
-  regexp *re_pan = regcomp( "^[ \011]*panelize[ \011]*=[ \011]*([^,]*)[ \011]*,(.*)$" );
 
-  ASSERT( re_ux );
-  ASSERT( re_see );
-  ASSERT( re_pan );
-  
-  #define opti(key,target) if(set_set(line, key, target)) continue
-  
+  VRegexp re_ux("^\\s*u?x\\s*=\\s*([^,]*)[ \011]*,\\s*([^, \011]*)\\s*,\\s*([^, \011]*)\\s*,(.*)$", "i");
+  VRegexp re_see( "^\\s*see\\s*=\\s*([^, \011]*)\\s*,(.*)$", "i" );
+  VRegexp re_pan( "^\\s*panelize\\s*=\\s*([^,]*)\\s*,(.*)$", "i" );
+
   char line[1024];
   if ( (fsett = fopen( filename_conf, "r")) )
     {
@@ -385,89 +342,74 @@ void vfu_settings_load()
       str_cut( line, "\n\r" );
       if ( strlen( line ) == 0 ) continue;
       
-      opti("browser", shell_browser);
-      opti("pager",   shell_browser);
-      opti("viewer",  shell_browser);
+      if(set_str( line, "browser", shell_browser))continue;
+      if(set_str( line, "pager", shell_browser))continue;
+      if(set_str( line, "viewer", shell_browser))continue;
 
-      opti("archive",  &archive_extensions);
+      if(set_arr( line, "archive",  archive_extensions))continue;
       
-      opti("editor", shell_editor);
+      if(set_str( line, "editor", shell_editor))continue;
       
-      opti("bookmark1", path_bookmarks[1]);
-      opti("bookmark2", path_bookmarks[2]);
-      opti("bookmark3", path_bookmarks[3]);
-      opti("bookmark4", path_bookmarks[4]);
-      opti("bookmark5", path_bookmarks[5]);
-      opti("bookmark6", path_bookmarks[6]);
-      opti("bookmark7", path_bookmarks[7]);
-      opti("bookmark8", path_bookmarks[8]);
-      opti("bookmark9", path_bookmarks[9]);
+      if(set_arr( line, "bookmark",  path_bookmarks))continue;
+      // follow 10 are deprecated
+      if(set_arr( line, "bookmark1", path_bookmarks))continue;
+      if(set_arr( line, "bookmark2", path_bookmarks))continue;
+      if(set_arr( line, "bookmark3", path_bookmarks))continue;
+      if(set_arr( line, "bookmark4", path_bookmarks))continue;
+      if(set_arr( line, "bookmark5", path_bookmarks))continue;
+      if(set_arr( line, "bookmark6", path_bookmarks))continue;
+      if(set_arr( line, "bookmark7", path_bookmarks))continue;
+      if(set_arr( line, "bookmark8", path_bookmarks))continue;
+      if(set_arr( line, "bookmark9", path_bookmarks))continue;
             
-   /* opti("cBLACK"   , ext_colors[0]); */
-      opti("cGREEN"   , ext_colors[cGREEN]);
-      opti("cRED"     , ext_colors[cRED]);
-      opti("cCYAN"    , ext_colors[cCYAN]);
-      opti("cWHITE"   , ext_colors[cWHITE]);
-      opti("cMAGENTA" , ext_colors[cMAGENTA]);
-      opti("cBLUE"    , ext_colors[cBLUE]);
-      opti("cYELLOW"  , ext_colors[cYELLOW]);
-      opti("chBLACK"  , ext_colors[chBLACK]);
-      opti("chGREEN"  , ext_colors[chGREEN]);
-      opti("chRED"    , ext_colors[chRED]);
-      opti("chCYAN"   , ext_colors[chCYAN]);
-      opti("chWHITE"  , ext_colors[chWHITE]);
-      opti("chMAGENTA", ext_colors[chMAGENTA]);
-      opti("chBLUE"   , ext_colors[chBLUE]);
-      opti("chYELLOW" , ext_colors[chYELLOW]);
+   /* if(set_str( line, "cblack"   , ext_colors[0]); */
+      if(set_str( line, "cgreen"   , ext_colors[cGREEN]))continue;
+      if(set_str( line, "cred"     , ext_colors[cRED]))continue;
+      if(set_str( line, "ccyan"    , ext_colors[cCYAN]))continue;
+      if(set_str( line, "cwhite"   , ext_colors[cWHITE]))continue;
+      if(set_str( line, "cmagenta" , ext_colors[cMAGENTA]))continue;
+      if(set_str( line, "cblue"    , ext_colors[cBLUE]))continue;
+      if(set_str( line, "cyellow"  , ext_colors[cYELLOW]))continue;
+      if(set_str( line, "chblack"  , ext_colors[chBLACK]))continue;
+      if(set_str( line, "chgreen"  , ext_colors[chGREEN]))continue;
+      if(set_str( line, "chred"    , ext_colors[chRED]))continue;
+      if(set_str( line, "chcyan"   , ext_colors[chCYAN]))continue;
+      if(set_str( line, "chwhite"  , ext_colors[chWHITE]))continue;
+      if(set_str( line, "chmagenta", ext_colors[chMAGENTA]))continue;
+      if(set_str( line, "chblue"   , ext_colors[chBLUE]))continue;
+      if(set_str( line, "chyellow" , ext_colors[chYELLOW]))continue;
     
-      opti("TrimTree",   trim_tree  );
+      if(set_splitter( line, "trimtree",  trim_tree  ))continue;
   
       /* following code is used to clean input data */
-      if( regexec( re_ux, line ) )
+      if( re_ux.m( line ) )
         {
         str = "";
-        regsubn( re_ux, 1, t ); /* get description */
+        str = str + re_ux[1] + ","; /* get description */
+        str = str + re_ux[2] + ","; /* get key name */
+        VString t = re_ux[3]; /* get extensions */
+        if ( t != "*" && t[-1] != '.' ) t += ".";
         str = str + t + ",";
-        regsubn( re_ux, 2, t ); /* get key name */
-        str = str + t + ",";
-        regsubn( re_ux, 3, t ); /* get extensions */
-        if ( strlen( t ) > 0 && strcmp( t, "*" ) != 0 )
-          if ( str_get_ch( t, -1 ) != '.' ) 
-            str_add_ch( t, '.' );
-        str = str + t + ",";
-        regsubn( re_ux, 4, t ); /* get shell line */
-        str += t;
-        user_externals.add( str );
+        str += re_ux[4]; /* get shell line */
+        user_externals.push( str );
         continue;
         } else
-      if ( regexec( re_see, line ) )
+      if( re_see.m( line ) )
         {
         str = "";
-        regsubn( re_see, 1, t ); /* get fnmatch mask */
-        str = str + t + ",";
-        regsubn( re_see, 2, t ); /* get shell line */
-        str += t;
-        see_filters.add( str );
+        see_filters.push( str + re_see[1] + "," + re_see[2] );
         continue;
         } else
-      if ( regexec( re_pan, line ) )
+      if( re_pan.m( line ) )
         {
         str = "";
-        regsubn( re_pan, 1, t ); /* get description */
-        str = str + t + ",";
-        regsubn( re_pan, 2, t ); /* get shell line */
-        str += t;
-        panelizers.add( str );
+        panelizers.push( str + re_see[1] + "," + re_see[2] );
         continue;
         }
       }
     fclose(fsett);
     }
   
-  free( re_ux );
-  free( re_see );
-  free( re_pan );
-    
   #ifdef _TARGET_GO32_
   int z;
   for ( z = 0; z < 16; z++ ) str_low( ext_colors[z] );
@@ -476,7 +418,7 @@ void vfu_settings_load()
   if (opt.use_dir_colors) vfu_load_dir_colors();
 //  if (file_load_crc32( filename_size_cache, &size_cache, sizeof(size_cache)))
 //    memset( &size_cache, 0, sizeof(size_cache) );
-  size_cache.zap();
+  size_cache.undef();
   size_cache.fload( filename_size_cache );
 }
 
@@ -486,7 +428,7 @@ void vfu_settings_save()
 {
   file_save_crc32( filename_opt, &opt, sizeof(opt));
 //  file_save_crc32( filename_size_cache, &size_cache, sizeof(size_cache));
-  SaveToFile( filename_history, &history );
+  history.fsave( filename_history );
   size_cache.fsave( filename_size_cache );
 }
 
@@ -513,10 +455,10 @@ void vfu_edit_conf_file()
     }
   else
     {
-    String line = shell_editor;
+    VString line = shell_editor;
     str_replace( line, "%f", filename_conf );
     str_replace( line, "%F", filename_conf );
-    vfu_shell( line.asis(), 0 );
+    vfu_shell( line.data(), 0 );
     }
   vfu_settings_save();
   vfu_settings_load();
@@ -530,7 +472,8 @@ void vfu_edit_conf_file()
 void vfu_options()
 {
   vfu_toggle_box( 30, 5, "Options/Toggles   (scroll down...)", Toggles );
-  file_list_index.type = opt.dynamic_scroll;
+  vfu_settings_save();
+  vfu_settings_load();
   vfu_drop_all_views();
   vfu_redraw(); 
   vfu_redraw_status();
