@@ -1,10 +1,12 @@
 /*
  *
- * (c) Vladi Belperchinov-Shabanski "Cade" 1996-2000
- * http://www.biscom.net/~cade  <cade@biscom.net>  <cade@datamax.bg>
+ * (c) Vladi Belperchinov-Shabanski "Cade" 1996-2003
+ * http://soul.datamax.bg/~cade  <cade@biscom.net>  <cade@datamax.bg>
  *
  * SEE `README',`LICENSE' OR `COPYING' FILE FOR LICENSE AND OTHER DETAILS!
  *
+ * $Id: vfuopt.h,v 1.8 2003/01/29 22:59:16 cade Exp $
+ *
  */
 
 #ifndef _VFUOPT_H_
@@ -35,10 +37,9 @@
     int f_time_type;
 
     int long_name_view;
-    int tree_auto;
     int tree_compact;
-    int tree_sizes;
     int tree_cd;
+    int fast_size_cache;
 
     int show_hidden_files; /* `dot' files in UNIX, `HS' files in dos */
     
@@ -46,6 +47,7 @@
 
     int use_colors;
     int use_dir_colors; /* /etc/DIR_COLORS */
+    int lower_case_ext_config;
 
     int copy_free_space_check;
     int copy_calc_totals;
@@ -65,7 +67,6 @@
     int menu_borders;
 
     int lynx_navigation; /* should <- == - and -> == + */
-    int dynamic_scroll;
 
     int auto_mount;
     int keep_selection; /* on rescan files */
@@ -85,9 +86,9 @@
   time_t vfu_opt_time( time_t ctime, time_t mtime, time_t atime );
   
   int set_set( const char *line, const char *keyword, char *target );
-  int set_set( const char *line, const char *keyword, String &target );
+  int set_set( const char *line, const char *keyword, VString &target );
   int set_set( const char *line, const char *keyword, int &target );
-  int set_set( const char *line, const char *keyword, StrSplitter &splitter );
+  int set_set( const char *line, const char *keyword, VArray &splitter );
   
   void vfu_settings_load();
   void vfu_settings_save();
