@@ -1,10 +1,12 @@
 /*
  *
- * (c) Vladi Belperchinov-Shabanski "Cade" 1996-2000
- * http://www.biscom.net/~cade  <cade@biscom.net>  <cade@datamax.bg>
+ * (c) Vladi Belperchinov-Shabanski "Cade" 1996-2003
+ * http://soul.datamax.bg/~cade  <cade@biscom.net>  <cade@datamax.bg>
  *
  * SEE `README',`LICENSE' OR `COPYING' FILE FOR LICENSE AND OTHER DETAILS!
  *
+ * $Id: vfusys.cpp,v 1.8 2003/01/29 22:59:16 cade Exp $
+ *
  */
       
 #include "vfu.h"
@@ -18,8 +20,6 @@
 
 /*###########################################################################*/
 
-#ifdef _TARGET_UNIX_
-
 void file_get_mode_str( const mode_t tm, mode_str_t &mod_str )
 {
   strcpy( mod_str, MODE_OFF );
@@ -40,11 +40,14 @@ void file_get_mode_str( const mode_t tm, mode_str_t &mod_str )
   if ((tm & S_IWOTH) != 0) mod_str[8] = 'w';
   if ((tm & S_IXOTH) != 0) mod_str[9] = 'x';
 
+  #ifndef _TARGET_GO32_
   if ((tm & S_ISUID) != 0) mod_str[3] = ((tm & S_IXUSR) != 0) ? 's' : 'S';
   if ((tm & S_ISGID) != 0) mod_str[6] = ((tm & S_IXGRP) != 0) ? 's' : 'S';
   if ((tm & S_ISVTX) != 0) mod_str[9] = ((tm & S_IXOTH) != 0) ? 't' : 'T';
+  #endif
+  
   #ifdef _TARGET_GO32_
-    mod_str[4]=mod_str[5]=mod_str[6]=mod_str[7]=mod_str[8]=mod_str[9]='-';
+  mod_str[4]=mod_str[5]=mod_str[6]=mod_str[7]=mod_str[8]=mod_str[9]='-';
   #endif
 }
 
@@ -93,12 +96,14 @@ int  file_set_mode_str( const char *filename, const mode_str_t mod_str )
   if (new_mod_str[8] == 'w') new_mode |= S_IWOTH;
   if (new_mod_str[9] == 'x') new_mode |= S_IXOTH;
   
+  #ifndef _TARGET_GO32_
   if (new_mod_str[3] == 's') { new_mode |= S_ISUID; new_mode |= S_IXUSR; }
   if (new_mod_str[3] == 'S') new_mode |= S_ISUID;
   if (new_mod_str[6] == 's') { new_mode |= S_ISGID; new_mode |= S_IXGRP; }
   if (new_mod_str[6] == 'S') new_mode |= S_ISGID;
   if (new_mod_str[9] == 't') { new_mode |= S_ISVTX; new_mode |= S_IXOTH; }
   if (new_mod_str[9] == 'T') new_mode |= S_ISVTX;
+  #endif
   
   return ( chmod( filename, new_mode ) != 0 );
 }
@@ -147,10 +152,10 @@ int  vfu_edit_attr( mode_str_t mod_str, int allow_masking )
   { ' ', "Read      Other", &mode_i[ 7], AONOFF },
   { ' ', "Write     Other", &mode_i[ 8], AONOFF },
   { ' ', "Exec/Srch Other", &mode_i[ 9], AONOFF },
-  { ' ', "Set user  id", &mode_i[10], AONOFF },
-  { ' ', "Set group id", &mode_i[11], AONOFF },
-  { ' ', "Sticky  Bit", &mode_i[12], AONOFF },
-  { -1, "---", NULL, NULL },
+  { ' ', "Set user  id",    &mode_i[10], AONOFF },
+  { ' ', "Set group id",    &mode_i[11], AONOFF },
+  { ' ', "Sticky  Bit",     &mode_i[12], AONOFF },
+  {  -1, "---",             NULL,        NULL },
   };
 
   if ( !vfu_toggle_box( 50, 5, "Change file Mode", mode_toggles ) ) return 0;
@@ -172,120 +177,6 @@ int  vfu_edit_attr( mode_str_t mod_str, int allow_masking )
   return 1;
 }
 
-#endif // _TARGET_UNIX_
-
-/*###########################################################################*/
-
-#ifdef _TARGET_GO32_
-
-void file_get_mode_str( const mode_t tm, mode_str_t &mod_str )
-{
-  strcpy( mod_str, MODE_OFF );
-  /* if (tm == -1) return;  _chmod(..0) failed */
-
-  if ((tm & MODE_DIR) != 0) mod_str[0] = 'D';
-  if ((tm & MODE_VOL) != 0) mod_str[1] = 'V';
-                            mod_str[2] = '-';
-                            mod_str[3] = '-';
-                            mod_str[4] = '-';
-                            mod_str[5] = '-';
-  if ((tm & MODE_RO ) != 0) mod_str[6] = 'R';
-  if ((tm & MODE_HID) != 0) mod_str[7] = 'H';
-  if ((tm & MODE_SYS) != 0) mod_str[8] = 'S';
-  if ((tm & MODE_ARC) != 0) mod_str[9] = 'A';
-};
-
-/*---------------------------------------------------------------------------*/
-
-int  file_get_mode_str( const char *filename, mode_str_t &mod_str )
-{
-  strcpy( mod_str, MODE_OFF );
-  mode_t mode = _chmod( filename, 0, 0 );
-  if ( mode == -1 ) return 1;
-  file_get_mode_str( mode, mod_str );
-  return 0;
-};
-
-/*---------------------------------------------------------------------------*/
-
-int  file_set_mode_str( const char *filename, const mode_str_t mod_str )
-{
-  char old_mod_str[12];
-  char new_mod_str[12];
-  mode_t new_mode = 0;
-  
-  
-  if (strchr( new_mod_str, '?' ))
-    {
-    if ( file_get_mode_str(filename, old_mod_str) ) return 1;
-    
-    strcpy( new_mod_str, mod_str );
-    
-    if (new_mod_str[1] == '?') new_mod_str[1] = old_mod_str[1];
-    if (new_mod_str[2] == '?') new_mod_str[2] = old_mod_str[2];
-    if (new_mod_str[3] == '?') new_mod_str[3] = old_mod_str[3];
-    if (new_mod_str[4] == '?') new_mod_str[4] = old_mod_str[4];
-    if (new_mod_str[5] == '?') new_mod_str[5] = old_mod_str[5];
-    if (new_mod_str[6] == '?') new_mod_str[6] = old_mod_str[6];
-    if (new_mod_str[7] == '?') new_mod_str[7] = old_mod_str[7];
-    if (new_mod_str[8] == '?') new_mod_str[8] = old_mod_str[8];
-    if (new_mod_str[9] == '?') new_mod_str[9] = old_mod_str[9];
-    
-    new_mod_str[0] = '-'; /* MODE_DIR -- must be off! */
-    new_mod_str[1] = '-'; /* MODE_VOL -- must be off! */
-    }
-    
-  if (new_mod_str[6] == 'R') new_mode |= MODE_RO ;
-  if (new_mod_str[7] == 'H') new_mode |= MODE_HID;
-  if (new_mod_str[8] == 'S') new_mode |= MODE_SYS;
-  if (new_mod_str[9] == 'A') new_mode |= MODE_ARC;
-  
-  return ( _chmod( filename, 1, new_mode ) == -1);
-};
-
-/*---------------------------------------------------------------------------*/
-
-int  vfu_edit_attr( mode_str_t mod_str, int allow_masking = 1 )
-{
-  int mode_i[16];
-  if (!allow_masking)
-    {
-    mode_i[6] = (mod_str[6] != 'R');
-    mode_i[7] = (mod_str[7] != 'H');
-    mode_i[8] = (mod_str[8] != 'S');
-    mode_i[9] = (mod_str[9] != 'A');
-    }
-  else
-    {
-    mode_i[6] = 2;
-    mode_i[7] = 2;
-    mode_i[8] = 2;
-    mode_i[9] = 2;
-    }
-
-  char* AONOFF1[] = { "YES", " - ", " ? ", NULL };
-  char* AONOFF2[] = { "YES", " - ", NULL };
-  #define AONOFF  (allow_masking ? AONOFF1 : AONOFF2)
-  ToggleEntry mode_toggles[] =
-  {
-  { 'R', "Read-only", &mode_i[6], AONOFF },
-  { 'H', "Hidden   ", &mode_i[7], AONOFF },
-  { 'S', "System   ", &mode_i[8], AONOFF },
-  { 'A', "Archive  ", &mode_i[9], AONOFF },
-  { -1, "---", NULL, NULL },
-  };
-
-  if ( !vfu_toggle_box( 50, 5, "Attributes/Mode", mode_toggles ) ) return 0;
-
-  if (mode_i[6] < 2) mod_str[6] = mode_i[6] == 0 ? 'R' : '-';
-  if (mode_i[7] < 2) mod_str[7] = mode_i[7] == 0 ? 'H' : '-';
-  if (mode_i[8] < 2) mod_str[8] = mode_i[8] == 0 ? 'S' : '-';
-  if (mode_i[9] < 2) mod_str[9] = mode_i[9] == 0 ? 'A' : '-';
-
-  return 1;
-}
-#endif /* _TARGET_GO32_ */
-
 /*---------------------------------------------------------------------------*/
 
 #ifdef _TARGET_GO32_
