@@ -1,10 +1,12 @@
 /*
  *
- * (c) Vladi Belperchinov-Shabanski "Cade" 1996-2000
- * http://www.biscom.net/~cade  <cade@biscom.net>  <cade@datamax.bg>
+ * (c) Vladi Belperchinov-Shabanski "Cade" 1996-2003
+ * http://soul.datamax.bg/~cade  <cade@biscom.net>  <cade@datamax.bg>
  *
  * SEE `README',`LICENSE' OR `COPYING' FILE FOR LICENSE AND OTHER DETAILS!
  *
+ * $Id: vfusys.h,v 1.5 2003/01/26 21:48:42 cade Exp $
+ *
  */
 
 #ifndef _VFUSYS_H_
@@ -14,25 +16,11 @@
 #include <sys/stat.h>
 #include <unistd.h>
 
-  #define MODE_OFF      "----------"
+#define MODE_OFF      "----------"
   
-#ifdef _TARGET_GO32_
-  #define MODE_STRING   "DV----RHSA"
-  #define MODE_MASK     "------????"
-  #define MODE_WRITE_ON "??????---?" /* allow WRITE mask */
-  #define MODE_RO    1
-  #define MODE_HID   2
-  #define MODE_SYS   4
-  #define MODE_VOL   8
-  #define MODE_DIR  16
-  #define MODE_ARC  32
-#endif
-
-#ifdef _TARGET_UNIX_
-  #define MODE_STRING   "drwxrwxrwx"
-  #define MODE_MASK     "-?????????"
-  #define MODE_WRITE_ON "??w???????" /* allow WRITE mask */
-#endif
+#define MODE_STRING   "drwxrwxrwx"
+#define MODE_MASK     "-?????????"
+#define MODE_WRITE_ON "??w???????" /* allow WRITE mask */
 
 typedef char mode_str_t[12];
 
