@@ -1,10 +1,12 @@
 /*
  *
- * (c) Vladi Belperchinov-Shabanski "Cade" 1996-2000
- * http://www.biscom.net/~cade  <cade@biscom.net>  <cade@datamax.bg>
+ * (c) Vladi Belperchinov-Shabanski "Cade" 1996-2003
+ * http://soul.datamax.bg/~cade  <cade@biscom.net>  <cade@datamax.bg>
  *
  * SEE `README',`LICENSE' OR `COPYING' FILE FOR LICENSE AND OTHER DETAILS!
  *
+ * $Id: vfutools.cpp,v 1.10 2003/01/26 21:48:42 cade Exp $
+ *
  */
 
 #include "vfumenu.h"
@@ -20,17 +22,17 @@ void __get_classify_str( const char *fname, char ch, char *tmp )
   tmp[0] = 0;
   if (ch == 'N')
     {
-    str_file_name( fname, tmp );
+    strcpy( tmp, str_file_name( fname ) );
     if (strlen(fname) == strlen(tmp)) strcat( tmp, ".---" );
     } else
   if (ch == 'E')
     {
-    str_file_ext( fname, tmp );
+    strcpy( tmp, str_file_ext( fname ) );
     if (strlen(tmp) == 0) strcat( tmp, "---" );
     }
   else
     {
-    str_file_name( fname, tmp );
+    strcpy( tmp, str_file_name( fname ) );
     str_sleft( tmp, ch - '0' );
     if (strlen(fname) == strlen(tmp)) strcat( tmp, ".---" );
     }
@@ -47,22 +49,22 @@ void vfu_tool_classify()
     return;
     }
 
-  mb.freeall();
-  mb.add("N Name");
-  mb.add("E Ext");
-  mb.add("1 First 1 letter");
-  mb.add("2 First 2 letters");
-  mb.add("3 First 3 letters");
-  mb.add("4 First 4 letters");
-  mb.add("5 First 5 letters");
-  mb.add("6 First 6 letters");
-  mb.add("7 First 7 letters");
-  mb.add("8 First 8 letters");
-  mb.add("9 First 9 letter");
+  mb.undef();
+  mb.push("N Name");
+  mb.push("E Ext");
+  mb.push("1 First 1 letter");
+  mb.push("2 First 2 letters");
+  mb.push("3 First 3 letters");
+  mb.push("4 First 4 letters");
+  mb.push("5 First 5 letters");
+  mb.push("6 First 6 letters");
+  mb.push("7 First 7 letters");
+  mb.push("8 First 8 letters");
+  mb.push("9 First 9 letter");
   if ( vfu_menu_box( 50, 5, "Classify files by") == -1 ) return;
   char ch = menu_box_info.ec;
 
-  mb.freeall();
+  mb.undef();
   int z;
   int i;
   for ( z = 0; z < files_count; z++ )
@@ -74,7 +76,7 @@ void vfu_tool_classify()
     int found = 0;
     for ( i = 0; i < mb.count(); i++ )
       if ( pathncmp( tmp, mb[i], tl ) == 0 ) { found = 1; break; }
-    if (!found) mb.add(tmp);
+    if (!found) mb.push(tmp);
     }
   for ( i = 0; i < mb.count(); i++ )
     {
@@ -82,7 +84,7 @@ void vfu_tool_classify()
     if ( res && !dir_exist( mb[i] ) )
       {
       char t[MAX_PATH];
-      sprintf( t, "Cannot create directory: %s (press a key)", mb[i] );
+      sprintf( t, "Cannot create directory: %s (press a key)", mb.get(i) );
       say1( t );
       say2errno();
       con_getch();
@@ -111,24 +113,25 @@ void vfu_tool_rename()
 {
   int z;
   int err;
-  String new_name;
-  char t[MAX_PATH];
+  VString path;
+  VString new_name;
+  VString t;
 
   if ( files_count < 1 )
     { say1( "No files to rename... (Empty directory)" ); return; };
   if ( sel_count < 1 )
     { say1( "No files to rename... (You have to select required files)" ); return; };
 
-  mb.freeall();
-  mb.add( "1 README.TXT => readme.txt" );
-  mb.add( "2 README.TXT => readme.TXT" );
-  mb.add( "3 README.TXT => README.txt" );
-  mb.add( "4 readme.txt => README.TXT" );
-  mb.add( "5 readme.txt => README.txt" );
-  mb.add( "6 readme.txt => readme.TXT" );
-  mb.add( "_ Replace spaces with _" );
-  mb.add( "Y Simplify name (RTFM)" );
-  mb.add( "S Sequential rename" );
+  mb.undef();
+  mb.push( "1 README.TXT => readme.txt" );
+  mb.push( "2 README.TXT => readme.TXT" );
+  mb.push( "3 README.TXT => README.txt" );
+  mb.push( "4 readme.txt => README.TXT" );
+  mb.push( "5 readme.txt => README.txt" );
+  mb.push( "6 readme.txt => readme.TXT" );
+  mb.push( "_ Replace spaces with _" );
+  mb.push( "Y Simplify name (RTFM)" );
+  mb.push( "S Sequential rename" );
   if (vfu_menu_box( 50, 5, "Rename Tools" ) == -1) return;
   switch( menu_box_info.ec )
     {
@@ -146,19 +149,19 @@ void vfu_tool_rename()
                  {
                  TF* fi = files_list[z];
                  
-                 if ( fi->is_dir() ) continue;
+                 // if ( fi->is_dir() ) continue; // why not? ;)
                  if ( !fi->sel ) continue;
-                 str_file_path( fi->name(), t ); /* FIXME: full name? */
-                 new_name = t;
+                 path = str_file_path( fi->name() );
+                 new_name = "";
                  
-                 str_file_name( fi->name(), t );
+                 t = str_file_name( fi->name() );
                  if (menu_box_info.ec == '1' || menu_box_info.ec == '2')
                    str_low( t );
                  if (menu_box_info.ec == '4' || menu_box_info.ec == '5')
                    str_up( t );
                  new_name += t;
                  
-                 str_file_ext( fi->name(), t );
+                 t = str_file_ext( fi->name() );
                  if (menu_box_info.ec == '1' || menu_box_info.ec == '3')
                    str_low( t );
                  if (menu_box_info.ec == '4' || menu_box_info.ec == '6')
@@ -175,11 +178,14 @@ void vfu_tool_rename()
                  
                  if (menu_box_info.ec == 'Y')
                    {
-                   str_tr( new_name, " `'&\"\\/,", "________" );
+                   str_replace( new_name, "%20", "_" );
+                   str_tr( new_name, " `'&\"\\/,()", "__________" );
                    str_squeeze( new_name, "_" );
                    str_replace( new_name, "_-_", "-" );
                    }
                  
+                 new_name = path + new_name;
+                 
                  if ( !file_exist( new_name) )
                    {
                    if (rename( fi->name(), new_name ) == 0) /* FIXME: full name ? */
@@ -204,10 +210,10 @@ void vfu_tool_rename()
 
 void vfu_tool_seq_rename()
 {
-  String prefix;
-  String suffix;
-  String s_digpos;
-  String s_start;
+  VString prefix;
+  VString suffix;
+  VString s_digpos;
+  VString s_start;
   
   if(!vfu_get_str( "Enter filename prefix: ", prefix, HID_SEQ_PREFIX )) return;
   if(!vfu_get_str( "Enter filename suffix: ", suffix, HID_SEQ_SUFFIX )) return;
@@ -220,9 +226,9 @@ void vfu_tool_seq_rename()
   if (digpos < 1 || digpos > 20) digpos = 1;
   if (start < 0) start = 0;
 
-  String new_name;
-  char t[MAX_PATH];
-  String fmt;
+  VString new_name;
+  VString t;
+  VString fmt;
   
   sprintf( fmt, "%%s%%0%dd%%s", digpos );
 
@@ -237,7 +243,7 @@ void vfu_tool_seq_rename()
     
     sprintf( new_name, fmt, prefix.data(), start, suffix.data() );
     
-    str_file_path( fi->name(), t ); /* FIXME: full name? */
+    t = str_file_path( fi->name() ); /* FIXME: full name? */
     new_name = t + new_name;
     
     if (access( new_name, F_OK ) == 0) { err++; continue; }
