@@ -1,10 +1,12 @@
 /*
  *
- * (c) Vladi Belperchinov-Shabanski "Cade" 1996-2000
- * http://www.biscom.net/~cade  <cade@biscom.net>  <cade@datamax.bg>
+ * (c) Vladi Belperchinov-Shabanski "Cade" 1996-2003
+ * http://soul.datamax.bg/~cade  <cade@biscom.net>  <cade@datamax.bg>
  *
  * SEE `README',`LICENSE' OR `COPYING' FILE FOR LICENSE AND OTHER DETAILS!
  *
+ * $Id: vfuuti.cpp,v 1.15 2003/01/29 22:59:16 cade Exp $
+ *
  */
 
 #include "vfu.h"
@@ -17,10 +19,10 @@
 
 /*---------------------------------------------------------------------------*/
 
-int vfu_update_shell_line( String &a_line, String &a_options )
+int vfu_update_shell_line( VString &a_line, VString &a_options )
 {
-String out;
-String s;
+VString out;
+VString s;
 
 int i = 0;
 
@@ -72,11 +74,7 @@ while( a_line[i] )
       case 'e' : /* name only */
       case 'E' : /* extension only */
                  if ( a_line[i+1] == 'e' )
-                   {
-                   char t[MAX_PATH];
-                   str_file_name( s, t );    
-                   s = t;
-                   }
+                   s = str_file_name( s );
                  else
                    s = files_list[FLI]->ext(); 
                  out += s;
@@ -209,7 +207,7 @@ int vfu_ask( const char *prompt, const char *allowed )
 
 /*---------------------------------------------------------------------------*/
 
-String& vfu_expand_mask( String& mask )
+VString& vfu_expand_mask( VString& mask )
 {
   if ( str_count( mask, "*?" ) > 0 ) return mask;
   mask += "*";
@@ -240,11 +238,11 @@ char* time_str_compact( const time_t tim, char* buf )
 char* size_str_compact( const fsize_t siz, char* buf )
 {
   if ( siz < 1024 )
-    sprintf( buf, "%.0lfb", siz );
+    sprintf( buf, "%.0fb", siz );
   else if ( siz < 1024*1024 )
-    sprintf( buf, "%.0lfk", siz/1024 );
+    sprintf( buf, "%.0fk", siz/1024 );
   else
-    sprintf( buf, "%.0lfm", siz/(1024*1024) );
+    sprintf( buf, "%.0fm", siz/(1024*1024) );
   return buf;
 }
 
@@ -257,12 +255,12 @@ void vfu_beep()
 
 /*###########################################################################*/
 
-#define MAXHIST         10      // max history items per id
+#define MAXHIST         14      // max history items per id
 #define HISTIDPAD       8
 
 void vfu_hist_add( int hist_id, const char* str )
 {
-  String hstr = hist_id;
+  VString hstr = hist_id;
   str_pad( hstr, HISTIDPAD );
   hstr += ",";
   hstr += str;
@@ -281,7 +279,7 @@ void vfu_hist_add( int hist_id, const char* str )
 
 const char* vfu_hist_get( int hist_id, int index )
 {
-  String hstr = hist_id;
+  VString hstr = hist_id;
   str_pad( hstr, HISTIDPAD );
   hstr += ",";
   int i = 0;
@@ -290,7 +288,7 @@ const char* vfu_hist_get( int hist_id, int index )
     if ( strncmp( hstr, history[z], HISTIDPAD+1 ) == 0 )
       {
       if ( index == -1 || index == i )
-        return history[z] + HISTIDPAD+1;
+        return history.get( z ) + HISTIDPAD+1;
       i++;
       }
   return NULL;
@@ -300,8 +298,7 @@ char* vfu_hist_get( int hist_id, int index, char* str )
 {
   str[0] = 0;
   const char* pstr = vfu_hist_get( hist_id, index );
-  if ( pstr )
-    strcpy( str, pstr );
+  if ( pstr ) strcpy( str, pstr );
   return str;
 };
 
@@ -317,7 +314,7 @@ int vfu_hist_index( int hist_id, const char* value )
 
 int vfu_hist_count( int hist_id )
 {
-  String hstr = hist_id;
+  VString hstr = hist_id;
   str_pad( hstr, HISTIDPAD );
   hstr += ",";
   int cnt = 0;
@@ -330,7 +327,7 @@ int vfu_hist_count( int hist_id )
 // use hist_id=-1 and/or index=-1 to remove all
 void vfu_hist_remove( int hist_id, int index )
 {
-  String hstr = hist_id;
+  VString hstr = hist_id;
   str_pad( hstr, HISTIDPAD );
   hstr += ",";
   int i = 0;
@@ -339,35 +336,35 @@ void vfu_hist_remove( int hist_id, int index )
     {
     if ( hist_id != -1 && strncmp( hstr, history[z], HISTIDPAD+1 ) != 0 ) { z++; continue; }
     if ( index != -1 && index != i ) { z++; i++; continue; }
-    history.free( z );
+    history.del( z );
     if ( index != -1 ) break;
     };
 };
 
+static char hist_menu_hotkeys[] = "1234567890ABCDEFGHIJKLMNOPQRSTUVWXYZ";
 int vfu_hist_menu( int x, int y, const char* title, int hist_id )
 {
-  String str;
+  VString str;
   
-  mb.freeall();
+  mb.undef();
   int z;
   int cnt = vfu_hist_count( hist_id );
   if ( cnt < 1 ) return -1;
   for ( z = 0; z < cnt; z++ )
     {
-    str = (z+1)%10;
+    ASSERT( z < str_len( hist_menu_hotkeys ) );
+    str = "";
+    str_add_ch( str, hist_menu_hotkeys[z] );
     str = str + " " + vfu_hist_get( hist_id, z );
-    mb.add( str );
+    mb.push( str );
     }
   return vfu_menu_box( x, y, title );
 };
 
-////////////////////////////////////////////////////////////////////////////
-//
-//
-//
+/*---------------------------------------------------------------------------*/
 
 int __vfu_get_str_hist_id; /* used to keep history id passed here... */
-void vfu_get_str_history( int key, String &s, int &pos )
+void vfu_get_str_history( int key, VString &s, int &pos )
 {
   if ( __vfu_get_str_hist_id <= 0 ) return;
   if ( key != KEY_NPAGE && key != KEY_PPAGE ) return;
@@ -377,12 +374,12 @@ void vfu_get_str_history( int key, String &s, int &pos )
 
   con_cshow();
   if ( z == -1 ) return;
-  s = mb[z] + 2;
+  s = mb.get(z) + 2;
   str_cut_spc( s );
   pos = str_len( s );
 }
 
-int vfu_get_str( const char *prompt, String& target, int hist_id, int x, int y )
+int vfu_get_str( const char *prompt, VString& target, int hist_id, int x, int y )
 {
   if ( x == -1 ) x = 1;
   if ( y == -1 ) y = con_max_y();
@@ -408,6 +405,19 @@ int vfu_get_str( const char *prompt, String& target, int hist_id, int x, int y )
   return ( r != 0 );  
 };
 
+/*---------------------------------------------------------------------------*/
+
+/* to fool gcc warning that mktemp() is not safe, I don't really care :/ 
+   still there is no mkstemp for directories :)) */
+char vfu_temp_filename[MAX_PATH];
+const char* vfu_temp()
+{
+    strcpy( vfu_temp_filename, tmp_path + "vfu.XXXXXX" );
+    mkstemp( vfu_temp_filename );
+    unlink( vfu_temp_filename );
+    return vfu_temp_filename;
+};
+
 /*###########################################################################*/
 
 /* eof vfuuti.cpp */
