@@ -1,10 +1,12 @@
 /*
  *
- * (c) Vladi Belperchinov-Shabanski "Cade" 1996-2000
- * http://www.biscom.net/~cade  <cade@biscom.net>  <cade@datamax.bg>
+ * (c) Vladi Belperchinov-Shabanski "Cade" 1996-2003
+ * http://soul.datamax.bg/~cade  <cade@biscom.net>  <cade@datamax.bg>
  *
  * SEE `README',`LICENSE' OR `COPYING' FILE FOR LICENSE AND OTHER DETAILS!
  *
+ * $Id: vfuuti.h,v 1.7 2003/01/26 21:48:42 cade Exp $
+ *
  */
 
 #ifndef _VFUUTI_H_
@@ -14,12 +16,12 @@
 
 /*###########################################################################*/
 
-int vfu_update_shell_line( String &a_line, String &a_options );
+int vfu_update_shell_line( VString &a_line, VString &a_options );
 int vfu_break_op(); /* return != 0 if ESC pressed, non blocking */
 int vfu_ask( const char *prompt, const char *allowed ); /* blocking */
 /* used before copy/move to calc estimated size */
 fsize_t vfu_update_sel_size( int one ); 
-String& vfu_expand_mask( String& mask );
+VString& vfu_expand_mask( VString& mask );
 char* time_str_compact( const time_t tim, char* buf );
 void vfu_beep();
 
@@ -35,8 +37,10 @@ int vfu_hist_count( int hist_id );
 void vfu_hist_remove( int hist_id, int index );
 int vfu_hist_menu( int x, int y, const char* title, int hist_id );
 
-void vfu_get_str_history( int key, String &s, int &pos ); /* internal! */
-int vfu_get_str( const char *prompt, String& target, int hist_id, int x = -1, int y = -1 );
+void vfu_get_str_history( int key, VString &s, int &pos ); /* internal! */
+int vfu_get_str( const char *prompt, VString& target, int hist_id, int x = -1, int y = -1 );
+
+const char* vfu_temp();
 
 /*###########################################################################*/
 
