@@ -1,10 +1,12 @@
 /*
  *
- * (c) Vladi Belperchinov-Shabanski "Cade" 1996-2000
- * http://www.biscom.net/~cade  <cade@biscom.net>  <cade@datamax.bg>
+ * (c) Vladi Belperchinov-Shabanski "Cade" 1996-2003
+ * http://soul.datamax.bg/~cade  <cade@biscom.net>  <cade@datamax.bg>
  *
  * SEE `README',`LICENSE' OR `COPYING' FILE FOR LICENSE AND OTHER DETAILS!
  *
+ * $Id: vfuview.cpp,v 1.14 2003/03/11 21:25:35 cade Exp $
+ *
  */
 
 #include "vfu.h"
@@ -19,7 +21,7 @@ int tag_mark_pos;
 
 int get_item_color( TF *fi )
 {
-  String str;
+  VString str;
 
   ASSERT( fi );
  
@@ -34,10 +36,11 @@ int get_item_color( TF *fi )
   else
     {
     str = fi->ext();
-    if ( str == "" )
-      str = ".";
+    if ( str == "" ) str = ".";
     }
   str += ".";
+  
+  if ( opt.lower_case_ext_config ) str_low( str ); // lowercase extension
 
   #ifdef _TARGET_GO32_
   /* under dos/windows file names are not case sensitive */
@@ -45,17 +48,23 @@ int get_item_color( TF *fi )
   #endif
  
   int z;
-  for ( z = cBLACK; z <= chWHITE; z++ )
-    if (str_find( ext_colors[z], str ) != -1)
-      return z;
+  if ( str != ".." )
+    {
+    for ( z = cBLACK; z <= chWHITE; z++ )
+      if (str_find( ext_colors[z], str ) != -1)
+        return z;
+    }    
 
   /* extension not found -- try type string */
   str = fi->type_str();
   str = "." + str + ".";
  
-  for ( z = cBLACK; z <= chWHITE; z++ )
-    if (str_find( ext_colors[z], str ) != -1)
-      return z;
+  if ( str != ".." )
+    {
+    for ( z = cBLACK; z <= chWHITE; z++ )
+      if (str_find( ext_colors[z], str ) != -1)
+        return z;
+    }    
 
   /* type string not found too return std color */
   return cNORMAL;
@@ -66,8 +75,8 @@ int get_item_color( TF *fi )
 void show_pos( int curr, int all )
 {
   char t[64];
-  sprintf( t, "%5d of %5d ", curr, all );
-  con_out( con_max_x() - 14, 3, t, cHEADER );
+  sprintf( t, "%5d of %5d", curr, all );
+  con_out( con_max_x() - 13, 3, t, cHEADER );
 }
 
 /*#######################################################################*/
@@ -90,7 +99,7 @@ void vfu_draw( int n )
   TF* fi = files_list[n];
  
   int c = fi->color(); /* color to be used */
-  String view = fi->view();
+  VString view = fi->view();
   if ( fi->sel )
     {
     str_set_ch( view, sel_mark_pos, '#' );
@@ -114,7 +123,7 @@ extern char *FTIMETYPE[]; /* in vfuopt.cpp */
 void vfu_redraw() /* redraw file list and header */
 {
   char t[MAX_PATH];
-  String str;
+  VString str;
 
   str  = "Mask: ";
   str += files_mask;
@@ -134,10 +143,27 @@ void vfu_redraw() /* redraw file list and header */
     str += "]/"; /* NOTE: to simulate root dir visually */
     str += archive_path;
     }
-  str_dot_reduce( NULL, str, con_max_x()-1 );
+  str = str_dot_reduce( str, con_max_x()-1 );
   con_out(1,2,str,cINFO);
   con_ce(cINFO);
 
+  str = "";
+  
+  if ( opt.sort_order == 'N' ) str = "NAME";
+  if ( opt.sort_order == 'M' ) str = "NAME#";
+  if ( opt.sort_order == 'E' ) str = "EXT";
+  if ( opt.sort_order == 'A' ) str = "MODE";
+  if ( opt.sort_order == 'O' ) str = "OWNER";
+  if ( opt.sort_order == 'G' ) str = "GROUP";
+  if ( opt.sort_order == 'T' ) str = "MTIME";
+  if ( opt.sort_order == 'H' ) str = "CTIME";
+  if ( opt.sort_order == 'C' ) str = "ATIME";
+  if ( opt.sort_order == 'S' ) str = "SIZE";
+  if ( opt.sort_order == 'Y' ) str = "TYPE";
+  str += opt.sort_direction == 'A' ? "+" : "-";
+  str = "(SORT:" + str + ")";
+  con_out( con_max_x() - str_len( str ) + 1, 2, str, cHEADER );
+  
   str = "";
  
   t[0] = 0;
@@ -145,22 +171,19 @@ void vfu_redraw() /* redraw file list and header */
   if (opt.sort_order == 'D') opt.sort_order = 'T'; /* hack anyway */
   if (!opt.long_name_view)
     {
-    /* sort order char +/- */
-    #define SOCH(c) ((opt.sort_order == (c))?(opt.sort_direction == 'A'?'+':'-' ):' ')
-    if (opt.f_mode  ) spos += sprintf( spos, "%10s%c", MODE_STRING,  SOCH('A') );
-    if (opt.f_owner ) spos += sprintf( spos, "  %cOWNER ", SOCH('O') );
-    if (opt.f_group ) spos += sprintf( spos, "  %cGROUP ", SOCH('G') );
-    if (opt.f_time  ) spos += sprintf( spos, "%s %cTiME ", FTIMETYPE[opt.f_time_type], SOCH('T') );
-    if (opt.f_size  ) spos += sprintf( spos, "         %cSiZE ", SOCH('S') );
+    if (opt.f_mode  ) spos += sprintf( spos, "%10s ", MODE_STRING );
+    if (opt.f_owner ) spos += sprintf( spos, "   OWNER " );
+    if (opt.f_group ) spos += sprintf( spos, "   GROUP " );
+    if (opt.f_time  ) spos += sprintf( spos, "%s  TiME ", FTIMETYPE[opt.f_time_type] );
+    if (opt.f_size  ) spos += sprintf( spos, "          SiZE " );
     };
   if ( opt.f_mode + opt.f_owner + opt.f_group + opt.f_time + opt.f_size + opt.f_type == 0 )
     opt.f_type = 1; /* a hack really :) if all fields are off -- turn on type one */
-  if (opt.f_type || opt.long_name_view) spos += sprintf( spos, "TP%c", SOCH('Y') );
-  tag_mark_pos = strlen( t ) - 1;
+  if (opt.f_type || opt.long_name_view) spos += sprintf( spos, "TP" );
+  tag_mark_pos = strlen( t );
   sel_mark_pos = tag_mark_pos + 2;
 
-  spos += sprintf( spos, " #NAME%c   %s",
-                   ((opt.sort_order == 'M') || (opt.sort_order == 'N')) ? (opt.sort_direction == 'A'?'+':'-') : ' ',
+  spos += sprintf( spos, "  #NAME    %s",
                    opt.long_name_view ? "( long name view )" : "" );
 
   str_pad( t, - con_max_x() );
@@ -171,10 +194,10 @@ void vfu_redraw() /* redraw file list and header */
 
   int z;
 
-
   for ( z = 0; z < PS; z++ )
     {
-    if (FLP+z >= files_count )
+    ASSERT( FLP + z >= 0 );
+    if ( FLP + z >= files_count )
       {
       con_out( 1, z+4, "~", cPLAIN );
       con_ce( cPLAIN );
@@ -196,9 +219,9 @@ void vfu_redraw() /* redraw file list and header */
 
 void vfu_redraw_status() /* redraw bottom status, total,free,selected... */
 {
-  String s1;
-  String s2;
-  String tmp;
+  VString s1;
+  VString s2;
+  VString tmp;
  
   /* first line here */
   s1  = "Select :";
@@ -242,8 +265,8 @@ void vfu_redraw_status() /* redraw bottom status, total,free,selected... */
   s2 += tmp;
  
   tmp = fs_block_size; str_pad( tmp,5 ); s2 += " [" + tmp + "]";
-  sprintf( tmp,"  U: %s.%s@%s ", user_id_str.asis(), group_id_str.asis(), 
-                                 host_name_str.asis() ); s2 += tmp;
+  sprintf( tmp,"  U: %s.%s@%s ", user_id_str.data(), group_id_str.data(), 
+                                 host_name_str.data() ); s2 += tmp;
 
   str_pad( s1, - con_max_x() );
   str_pad( s2, - con_max_x() );
@@ -381,8 +404,10 @@ void vfu_nav_select()
 
 void vfu_nav_update_pos()
 {
- if ( FLI < 0 ) FLI = 0;
- if ( FLI > files_count - 1) FGO( files_count - 1 );
+ ASSERT( files_count >= 0 );
+ if ( FLI < 0 ) FGO( 0 );
+ if ( files_count == 0 ) FGO( 0 );
+ if ( files_count > 0 && FLI > files_count - 1 ) FGO( files_count - 1 );
 }
 
 /* eof vfuview.cpp */
