@@ -1,3 +1,13 @@
+/*
+ *
+ * (c) Vladi Belperchinov-Shabanski "Cade" <cade@biscom.net> 1998-2003
+ *
+ * SEE `README',LICENSE' OR COPYING' FILE FOR LICENSE AND OTHER DETAILS!
+ *
+ * $Id: conmenu.cpp,v 1.8 2003/01/29 22:59:27 cade Exp $
+ *
+ */
+
 #include <ctype.h>
 
 #include <unicon.h>
@@ -9,7 +19,7 @@ ConMenuInfo con_default_menu_info;
 
 int con_toggle_box( int x, int y, const char *title, ToggleEntry* toggles, ConMenuInfo *menu_info  )
 {
-  TScrollPos scroll;
+  ScrollPos scroll;
   int z;
 
   int w = -1;
@@ -33,12 +43,13 @@ int con_toggle_box( int x, int y, const char *title, ToggleEntry* toggles, ConMe
   if (h > count) h = count;
   if (h == 0) h = 1;
 
+  // FIXME: those should be fixed!!!
   if (x + w > con_max_x()) w = con_max_x() - x - 1;
   if (y + h > con_max_y()-4) h = con_max_y() - y - 5;
 
-  String str;
-  String str1;
-  String hots = "";
+  VString str;
+  VString str1;
+  VString hots = "";
   for(z = 0; z < count; z++)
     if (strncmp("--", toggles[z].name, 2))
       str_add_ch( hots, toggles[z].key );
@@ -58,40 +69,36 @@ int con_toggle_box( int x, int y, const char *title, ToggleEntry* toggles, ConMe
   con_out(x,y,str,menu_info->ti);
   y++;
 
-  scroll.type = 1;
   scroll.wrap = 1;
-  scroll.min = 0;
-  scroll.max = count-1;
-  scroll.pagesize = h;
-  scroll.pos = 0;
-  scroll.page = 0;
-  scroll.gotopos(0);
+  scroll.set_min_max( 0, count-1 );
+  scroll.set_pagesize( h );
+  scroll.go( 0 );
 
   while(1)
     {
-    for( z = 0; z < scroll.pagesize; z++ )
+    for( z = 0; z < scroll.pagesize(); z++ )
       {
-      if (scroll.page+z >= count)
+      if (scroll.page() + z >= count)
         {
         str = " ~";
         str_pad( str, -(w+2), ' ');
         }
       else
         {
-        int sep = (strncmp("--", toggles[scroll.page+z].name, 2) == 0);
+        int sep = (strncmp("--", toggles[scroll.page()+z].name, 2) == 0);
         if (sep)
           {
           str = "";
-          str += toggles[scroll.page+z].name;
+          str += toggles[scroll.page()+z].name;
           str_pad( str, -w, '-');
           str += "--";
           }
         else
           {
-          str = " "; str_add_ch( str, toggles[scroll.page+z].key ); str += " ";
-          str += toggles[scroll.page+z].name;
+          str = " "; str_add_ch( str, toggles[scroll.page()+z].key ); str += " ";
+          str += toggles[scroll.page()+z].name;
           str_pad( str, -(w-6), ' ');
-          str1 = toggles[scroll.page+z].states[*(toggles[scroll.page+z].data)];
+          str1 = toggles[scroll.page()+z].states[*(toggles[scroll.page()+z].data)];
           str_pad( str1, 6, ' '); str1 += " ";
           str += " " + str1;
           }
@@ -102,29 +109,29 @@ int con_toggle_box( int x, int y, const char *title, ToggleEntry* toggles, ConMe
         str = " " + str + " ";
 //      if (str.len() > w) StrSLeft(str,w);
 //      str = " " + str + " ";
-      con_out( x, y+z, str, ( scroll.page+z != scroll.pos ) ? menu_info->cn : menu_info->ch );
+      con_out( x, y+z, str, ( scroll.page()+z != scroll.pos() ) ? menu_info->cn : menu_info->ch );
       }
     if (menu_info->bo)
       {
       str = "";
       str_pad( str, w+2, '-' );
       str = "`" + str + "'";
-      con_out( x, y+scroll.pagesize, str, menu_info->cn );
+      con_out( x, y+scroll.pagesize(), str, menu_info->cn );
       }
     ch = con_getch();
     menu_info->ec = ch;
 
     if ( ch == KEY_UP    ) scroll.up();
     if ( ch == KEY_DOWN  ) scroll.down();
-    if ( ch == KEY_NPAGE ) scroll.pagedown();
-    if ( ch == KEY_PPAGE ) scroll.pageup();
+    if ( ch == KEY_NPAGE ) scroll.npage();
+    if ( ch == KEY_PPAGE ) scroll.ppage();
     if ( ch == KEY_HOME  ) scroll.home();
     if ( ch == KEY_END   ) scroll.end();
 
     if ( ch < 0 || ch > 255 ) continue;
     if ( ch == 27 ) return 0;
     if ( ch == 13 /* && strncmp("--", toggles[scroll.pos].name, 2) */ ) return 1;
-    z = ( ch == ' ' ) ? scroll.pos : z = str_find( hots, ch );
+    z = ( ch == ' ' ) ? scroll.pos() : z = str_find( hots, ch );
     if (z > -1 && strncmp("--", toggles[z].name, 2) )
       {
       int state = *(toggles[z].data) + 1;
@@ -135,32 +142,33 @@ int con_toggle_box( int x, int y, const char *title, ToggleEntry* toggles, ConMe
   return -1;
 }
 
-int con_menu_box( int x, int y, const char *title, PSZCluster *sc, int hotkeys, ConMenuInfo *menu_info  )
+int con_menu_box( int x, int y, const char *title, VArray *va, int hotkeys, ConMenuInfo *menu_info  )
 {
-  TScrollPos scroll;
+  ScrollPos scroll;
   int z;
 
   int w = -1;
   int h = -1;
 
-  if (w == -1) w = (int)sc->maxlen();
-  if (h == -1) h = sc->count();
+  if (w == -1) w = va->max_len();
+  if (h == -1) h = va->count();
 
   z = strlen(title);
   if (w < z) w = z;
-  if (h > sc->count()) h = sc->count();
+  if (h > va->count()) h = va->count();
   if (h == 0) h = 1;
 
-  if (x + w > con_max_x()) w = con_max_x() - x - 5;
-  if (y + h > con_max_y()-5) h = con_max_y() - y - 5;
+  // FIXME: those should be fixed!!!
+  if (x + w > con_max_x()) w = con_max_x() - x - 4;
+  if (y + h > con_max_y()-4) h = con_max_y() - y - 4;
 
-  String str;
-  String hots = "";
+  VString str;
+  VString hots = "";
   if ( hotkeys > -1 )
     {
-    for(z = 0; z < sc->count(); z++)
-      if (strncmp("--", sc->get(z), 2))
-        str_add_ch( hots,int((const char*)(sc->get(z))[hotkeys]) );
+    for(z = 0; z < va->count(); z++)
+      if (strncmp("--", va->get(z), 2))
+        str_add_ch( hots,int((const char*)(va->get(z))[hotkeys]) );
       else
         str_add_ch( hots,' ' );
     str_up(hots);
@@ -180,18 +188,16 @@ int con_menu_box( int x, int y, const char *title, PSZCluster *sc, int hotkeys,
   con_out(x,y,str,menu_info->ti);
   y++;
 
-  scroll.type = 1;
   scroll.wrap = 1;
-  scroll.min = 0; scroll.max = sc->count()-1;
-  scroll.pagesize = h;
-  scroll.pos = 0; scroll.page = 0;
-  scroll.gotopos(0);
+  scroll.set_min_max( 0, va->count()-1 );
+  scroll.set_pagesize( h );
+  scroll.go( 0 );
 
   while(1)
     {
-    for( z = 0; z < scroll.pagesize; z++ )
+    for( z = 0; z < scroll.pagesize(); z++ )
       {
-      str = (scroll.page+z >= sc->count())? "~" : sc->get(scroll.page+z);
+      str = (scroll.page()+z >= va->count())? "~" : va->get(scroll.page()+z);
 
       if ( menu_info->hide_magic[0] )
         {
@@ -202,28 +208,27 @@ int con_menu_box( int x, int y, const char *title, PSZCluster *sc, int hotkeys,
  
       str_pad( str,-w , (strncmp("--", str, 2) == 0)?'-':' ');
       if (str_len(str) > w) 
-        // str_sleft(str,w);
-        str_dot_reduce( NULL, str, w );
+        str = str_dot_reduce( str, w );
       if (menu_info->bo)
         str = "| " + str + " |";
       else
         str = "  " + str + "  ";
-      con_out( x, y+z, str, ( scroll.page+z != scroll.pos ) ? menu_info->cn : menu_info->ch );
+      con_out( x, y+z, str, ( scroll.page()+z != scroll.pos() ) ? menu_info->cn : menu_info->ch );
       }
     if (menu_info->bo)
       {
       str = "";
       str_pad( str, w+2, '-' );
       str = "`" + str + "'";
-      con_out( x, y+scroll.pagesize, str, menu_info->cn );
+      con_out( x, y+scroll.pagesize(), str, menu_info->cn );
       }
     ch = con_getch();
     menu_info->ec = ch;
 
     if ( ch == KEY_UP    ) scroll.up();
     if ( ch == KEY_DOWN  ) scroll.down();
-    if ( ch == KEY_NPAGE ) scroll.pagedown();
-    if ( ch == KEY_PPAGE ) scroll.pageup();
+    if ( ch == KEY_NPAGE ) scroll.npage();
+    if ( ch == KEY_PPAGE ) scroll.ppage();
     if ( ch == KEY_HOME  ) scroll.home();
     if ( ch == KEY_END   ) scroll.end();
 
@@ -235,20 +240,20 @@ int con_menu_box( int x, int y, const char *title, PSZCluster *sc, int hotkeys,
       }
     if ( ch == 13 )
       {
-      if (strncmp("--", sc->get(scroll.pos), 2) != 0) // ako e "--" e separator
+      if (strncmp("--", va->get(scroll.pos()), 2) != 0) // ako e "--" e separator
         {
-        menu_info->ec = hots[scroll.pos];
+        menu_info->ec = hots[scroll.pos()];
         menu_info->ac = -1;
-        return scroll.pos;
+        return scroll.pos();
         }
       }
     if ( menu_info->ac > -1 && ch == menu_info->ac )
       {
-      if (strncmp("--", sc->get(scroll.pos), 2) != 0) // ako e "--" e separator
+      if (strncmp("--", va->get(scroll.pos()), 2) != 0) // ako e "--" e separator
         {
-        menu_info->ec = hots[scroll.pos];
+        menu_info->ec = hots[scroll.pos()];
         menu_info->ac = -2;
-        return scroll.pos;
+        return scroll.pos();
         }
       }
     z = str_find( hots, toupper(ch) );
@@ -263,17 +268,13 @@ int con_menu_box( int x, int y, const char *title, PSZCluster *sc, int hotkeys,
   return -1;
 };
 
-int con_full_psz_box( int x, int y, const char *title, PSZCluster *sc, ConMenuInfo *menu_info )
+int con_full_box( int x, int y, const char *title, VArray *va, ConMenuInfo *menu_info )
 {
-  TScrollPos scroll;
-  scroll.type = menu_info->st;
+  ScrollPos scroll;
   scroll.wrap = 0;
-  scroll.min = 0;
-  scroll.max = sc->count()-1;
-  scroll.pagesize = con_max_y() - 3; /* one title and two status */
-  scroll.pos = 0;
-  scroll.page = 0;
-  scroll.gotopos(0);
+  scroll.set_min_max( 0, va->count()-1 );
+  scroll.set_pagesize( con_max_y() - 3 ); /* one title and two status */
+  scroll.go( 0 );
 
   char pos_str[32];
 
@@ -282,31 +283,31 @@ int con_full_psz_box( int x, int y, const char *title, PSZCluster *sc, ConMenuIn
   con_ce( menu_info->ti );
   while(4)
     {
-    String str;
+    VString str;
     int z;
-    for( z = 0; z < scroll.pagesize; z++ )
+    for( z = 0; z < scroll.pagesize(); z++ )
       {
-      if ( scroll.page + z < sc->count() )
-        str = sc->get( scroll.page + z );
+      if ( scroll.page() + z < va->count() )
+        str = va->get( scroll.page() + z );
       else
         str = "~";
-      str_dot_reduce( NULL, str, con_max_x()-1 );
+      str = str_dot_reduce( str, con_max_x()-1 );
       con_xy( 1, z + 2 );
-      int c = ( scroll.page + z == scroll.pos ) ? menu_info->ch : menu_info->cn;
+      int c = ( scroll.page() + z == scroll.pos() ) ? menu_info->ch : menu_info->cn;
       con_puts( str, c );
       con_ce( c );
       }
-    sprintf( pos_str, " %5d of %5d", scroll.pos+1, scroll.max+1 );
+    sprintf( pos_str, " %5d of %5d", scroll.pos()+1, scroll.max()+1 );
     con_out( con_max_x() - 15, 1, pos_str, menu_info->ti );
     int ch;
     switch( (ch = con_getch()) )
       {
       case 27 : menu_info->ec = 27; return -1; break;
-      case 13 : menu_info->ec = 13; return scroll.pos; break;
+      case 13 : menu_info->ec = 13; return scroll.pos(); break;
       case KEY_UP    : scroll.up(); break;
       case KEY_DOWN  : scroll.down(); break;
-      case KEY_NPAGE : scroll.pagedown(); break;
-      case KEY_PPAGE : scroll.pageup(); break;
+      case KEY_NPAGE : scroll.npage(); break;
+      case KEY_PPAGE : scroll.ppage(); break;
       case KEY_HOME  : scroll.home(); break;
       case KEY_END   : scroll.end(); break;
       default: 
