@@ -1,9 +1,11 @@
 /*
  *
- * (c) Vladi Belperchinov-Shabanski "Cade" <cade@biscom.net> 1998-1999
+ * (c) Vladi Belperchinov-Shabanski "Cade" <cade@biscom.net> 1998-2003
  *
  * SEE `README',`LICENSE' OR `COPYING' FILE FOR LICENSE AND OTHER DETAILS!
  *
+ * $Id: unicon.cpp,v 1.4 2003/01/21 19:56:35 cade Exp $
+ *
  */
 
 #include "unicon.h"
@@ -78,7 +80,7 @@
   void con_puts( const char *s )
   {
     #ifdef _TARGET_GO32_
-    String str = s;
+    VString str = s;
     str_replace( str, "\n", "\r\n" );
     cputs( str );
     #else
